@@ -1,4 +1,4 @@
-static const char CVSID[] = "$Id: highlight.c,v 1.50 2004/08/01 10:06:10 yooden Exp $";
+static const char CVSID[] = "$Id: highlight.c,v 1.56 2008/01/04 22:11:03 yooden Exp $";
 /*******************************************************************************
 *									       *
 * highlight.c -- Nirvana Editor syntax highlighting (text coloring and font    *
@@ -141,22 +141,24 @@ static patternSet *findPatternsForWindow(WindowInfo *window, int warn);
 static highlightDataRec *compilePatterns(Widget dialogParent,
     	highlightPattern *patternSrc, int nPatterns);
 static void freePatterns(highlightDataRec *patterns);
-static void handleUnparsedRegion(WindowInfo* win, textBuffer *buf, int pos);
-static void handleUnparsedRegionCB(textDisp *textD, int pos, void *cbArg);
+static void handleUnparsedRegion(const WindowInfo* win, textBuffer* styleBuf,
+        const int pos);
+static void handleUnparsedRegionCB(const textDisp* textD, const int pos,
+        const void* cbArg);
 static void incrementalReparse(windowHighlightData *highlightData,
     	textBuffer *buf, int pos, int nInserted, const char *delimiters);
 static int parseBufferRange(highlightDataRec *pass1Patterns,
     	highlightDataRec *pass2Patterns, textBuffer *buf, textBuffer *styleBuf,
         reparseContext *contextRequirements, int beginParse, int endParse,
         const char *delimiters);
-static int parseString(highlightDataRec *pattern, char **string,
+static int parseString(highlightDataRec *pattern, const char **string,
     	char **styleString, int length, char *prevChar, int anchored,
-    	const char *delimiters, const char* lookBehindTo);
+    	const char *delimiters, const char* lookBehindTo, const char* match_till);
 static void passTwoParseString(highlightDataRec *pattern, char *string,
-    	char *styleString, int length, char *prevChar, int anchored,
-    	const char *delimiters, const char* lookBehindTo);
-static void fillStyleString(char **stringPtr, char **stylePtr, char *toPtr,
-    	char style, char *prevChar);
+        char *styleString, int length, char *prevChar, const char *delimiters,
+        const char* lookBehindTo, const char* match_till);
+static void fillStyleString(const char **stringPtr, char **stylePtr,
+        const char *toPtr, char style, char *prevChar);
 static void modifyStyleBuf(textBuffer *styleBuf, char *styleString,
     	int startPos, int endPos, int firstPass2Style);
 static int lastModified(textBuffer *styleBuf);
@@ -172,8 +174,8 @@ static int backwardOneContext(textBuffer *buf, reparseContext *context,
     	int fromPos);
 static int forwardOneContext(textBuffer *buf, reparseContext *context,
     	int fromPos);
-static void recolorSubexpr(regexp *re, int subexpr, int style, char *string,
-    	char *styleString);
+static void recolorSubexpr(regexp *re, int subexpr, int style,
+        const char *string, char *styleString);
 static int indexOfNamedPattern(highlightPattern *patList, int nPats,
     	const char *patName);
 static int findTopLevelParentIndex(highlightPattern *patList, int nPats,
@@ -204,7 +206,7 @@ static styleTableEntry *styleTableEntryOfCode(WindowInfo *window, int hCode);
 ** character typed.
 */
 void SyntaxHighlightModifyCB(int pos, int nInserted, int nDeleted,
-    	int nRestyled, char *deletedText, void *cbArg) 
+    	int nRestyled, const char *deletedText, void *cbArg) 
 {    
     WindowInfo *window = (WindowInfo *)cbArg;
     windowHighlightData 
@@ -258,7 +260,8 @@ void StartHighlighting(WindowInfo *window, int warn)
 {
     patternSet *patterns;
     windowHighlightData *highlightData;
-    char *stylePtr, *styleString, *stringPtr, *bufString;
+    char *stylePtr, *styleString;
+    const char  *stringPtr, *bufString;
     char prevChar = '\0';
     int i, oldFontHeight;
     
@@ -284,11 +287,10 @@ void StartHighlighting(WindowInfo *window, int warn)
     	for (i=0; i<window->buffer->length; i++)
     	    *stylePtr++ = UNFINISHED_STYLE;
     } else {
-	stringPtr = bufString = BufGetAll(window->buffer);
+	stringPtr = bufString = BufAsString(window->buffer);
 	parseString(highlightData->pass1Patterns, &stringPtr, &stylePtr,
     		window->buffer->length, &prevChar, False,
-    		GetWindowDelimiters(window), bufString);
-	XtFree(bufString);
+    		GetWindowDelimiters(window), bufString, NULL);
     }
     *stylePtr = '\0';
     BufSetAll(highlightData->styleBuffer, styleString);
@@ -1035,13 +1037,15 @@ static void freePatterns(highlightDataRec *patterns)
     	if (patterns[i].endRE != NULL)
     	    free((char *)patterns[i].endRE);
     	if (patterns[i].errorRE != NULL)
-    	    XtFree((char *)patterns[i].errorRE);
+    	    free((char *)patterns[i].errorRE);
     	if (patterns[i].subPatternRE != NULL)
     	    free((char *)patterns[i].subPatternRE);
     }
-    for (i=0; patterns[i].style!=0; i++)
-    	if (patterns[i].subPatterns != NULL)
-    	    XtFree((char *)patterns[i].subPatterns);
+
+    for (i=0; patterns[i].style!=0; i++) {
+        XtFree((char*) patterns[i].subPatterns);
+    }
+
     XtFree((char *)patterns);
 }
 
@@ -1205,18 +1209,6 @@ char *HighlightStyleOfCode(WindowInfo *window, int hCode)
     return entry ? entry->styleName : "";
 }
 
-char *HighlightColorOfCode(WindowInfo *window, int hCode)
-{
-    styleTableEntry *entry = styleTableEntryOfCode(window, hCode);
-    return entry ? entry->colorName : "";
-}
-
-char *HighlightBackgroundColorOfCode(WindowInfo *window, int hCode)
-{
-    styleTableEntry *entry = styleTableEntryOfCode(window, hCode);
-    return entry && entry->bgColorName ? entry->bgColorName : "";
-}
-
 Pixel HighlightColorValueOfCode(WindowInfo *window, int hCode,
       int *r, int *g, int *b)
 {
@@ -1277,18 +1269,6 @@ Pixel GetHighlightBGColorOfCode(WindowInfo *window, int hCode,
     }
 }
 
-int HighlightCodeIsBold(WindowInfo *window, int hCode)
-{
-    styleTableEntry *entry = styleTableEntryOfCode(window, hCode);
-    return entry ? entry->isBold : 0;
-}
-
-int HighlightCodeIsItalic(WindowInfo *window, int hCode)
-{
-    styleTableEntry *entry = styleTableEntryOfCode(window, hCode);
-    return entry ? entry->isItalic : 0;
-}
-
 /*
 ** Callback to parse an "unfinished" region of the buffer.  "unfinished" means
 ** that the buffer has been parsed with pass 1 patterns, but this section has
@@ -1298,8 +1278,8 @@ int HighlightCodeIsItalic(WindowInfo *window, int hCode)
 ** needs re-parsing.  This routine applies pass 2 patterns to a chunk of
 ** the buffer of size PASS_2_REPARSE_CHUNK_SIZE beyond pos.
 */
-static void handleUnparsedRegion(WindowInfo *window, textBuffer *styleBuf, 
-                                 int pos)
+static void handleUnparsedRegion(const WindowInfo* window, textBuffer* styleBuf,
+        const int pos)
 {
     textBuffer *buf = window->buffer;
     int beginParse, endParse, beginSafety, endSafety, p;
@@ -1308,7 +1288,8 @@ static void handleUnparsedRegion(WindowInfo *window, textBuffer *styleBuf,
 
     reparseContext *context = &highlightData->contextRequirements;
     highlightDataRec *pass2Patterns = highlightData->pass2Patterns;
-    char *string, *styleString, *stringPtr, *stylePtr, c, prevChar;
+    char *string, *styleString, *stylePtr, c, prevChar;
+    const char *stringPtr;
     int firstPass2Style = (unsigned char)pass2Patterns[1].style;
     
     /* If there are no pass 2 patterns to process, do nothing (but this
@@ -1356,13 +1337,13 @@ static void handleUnparsedRegion(WindowInfo *window, textBuffer *styleBuf,
     /* Copy the buffer range into a string */
     /* printf("callback pass2 parsing from %d thru %d w/ safety from %d thru %d\n",
     	    beginParse, endParse, beginSafety, endSafety); */
-    string = stringPtr = BufGetRange(buf, beginSafety, endSafety);
+    stringPtr = string = BufGetRange(buf, beginSafety, endSafety);
     styleString = stylePtr = BufGetRange(styleBuf, beginSafety, endSafety);
     
     /* Parse it with pass 2 patterns */
     prevChar = getPrevChar(buf, beginSafety);
     parseString(pass2Patterns, &stringPtr, &stylePtr, endParse - beginSafety,
-    	    &prevChar, False, GetWindowDelimiters(window), string);
+    	    &prevChar, False, GetWindowDelimiters(window), string, NULL);
 
     /* Update the style buffer the new style information, but only between
        beginParse and endParse.  Skip the safety region */
@@ -1376,9 +1357,10 @@ static void handleUnparsedRegion(WindowInfo *window, textBuffer *styleBuf,
 /*
 ** Callback wrapper around the above function.
 */
-static void handleUnparsedRegionCB(textDisp *textD, int pos, void *cbArg)
+static void handleUnparsedRegionCB(const textDisp* textD, const int pos,
+        const void* cbArg)
 {
-    handleUnparsedRegion((WindowInfo*)cbArg, textD->styleBuffer, pos);
+    handleUnparsedRegion((WindowInfo*) cbArg, textD->styleBuffer, pos);
 }
 
 /*
@@ -1483,7 +1465,8 @@ static int parseBufferRange(highlightDataRec *pass1Patterns,
         reparseContext *contextRequirements, int beginParse, int endParse,
         const char *delimiters)
 {
-    char *string, *styleString, *stringPtr, *stylePtr, *temp, prevChar;
+    char *string, *styleString, *stylePtr, *temp, prevChar;
+    const char *stringPtr;
     int endSafety, endPass2Safety, startPass2Safety, tempLen;
     int modStart, modEnd, beginSafety, beginStyle, p, style;
     int firstPass2Style = pass2Patterns == NULL ? INT_MAX :
@@ -1533,7 +1516,7 @@ static int parseBufferRange(highlightDataRec *pass1Patterns,
     stringPtr = &string[beginParse-beginSafety];
     stylePtr = &styleString[beginParse-beginSafety];
     parseString(pass1Patterns, &stringPtr, &stylePtr, endParse-beginParse,
-    	    &prevChar, False, delimiters, string);
+    	    &prevChar, False, delimiters, string, NULL);
 
     /* On non top-level patterns, parsing can end early */
     endParse = min(endParse, stringPtr-string + beginSafety);
@@ -1570,14 +1553,14 @@ static int parseBufferRange(highlightDataRec *pass1Patterns,
 	prevChar = getPrevChar(buf, beginSafety);
 	if (endPass2Safety == endSafety) {
 	    passTwoParseString(pass2Patterns, string, styleString,
-	    	endParse - beginSafety, &prevChar, False, delimiters, string);
+                    endParse - beginSafety, &prevChar, delimiters, string, NULL);
 	    goto parseDone;
 	} else {
 	    tempLen = endPass2Safety - modStart;
 	    temp = XtMalloc(tempLen);
 	    strncpy(temp, &styleString[modStart-beginSafety], tempLen);
 	    passTwoParseString(pass2Patterns, string, styleString,
-		    modStart - beginSafety, &prevChar, False, delimiters, string);
+                    modStart - beginSafety, &prevChar, delimiters, string, NULL);
 	    strncpy(&styleString[modStart-beginSafety], temp, tempLen);
 	    XtFree(temp);
 	}
@@ -1590,7 +1573,7 @@ static int parseBufferRange(highlightDataRec *pass1Patterns,
 	if (beginSafety > modEnd) {
 	    prevChar = getPrevChar(buf, beginSafety);
 	    passTwoParseString(pass2Patterns, string, styleString,
-	    	    endParse - beginSafety, &prevChar, False, delimiters, string);
+                    endParse - beginSafety, &prevChar, delimiters, string, NULL);
 	} else {
 	    startPass2Safety = max(beginSafety,
 	    	    backwardOneContext(buf, contextRequirements, modEnd));
@@ -1599,9 +1582,10 @@ static int parseBufferRange(highlightDataRec *pass1Patterns,
 	    strncpy(temp, &styleString[startPass2Safety-beginSafety], tempLen);
 	    prevChar = getPrevChar(buf, startPass2Safety);
 	    passTwoParseString(pass2Patterns,
-	    	    &string[startPass2Safety-beginSafety],
-	    	    &styleString[startPass2Safety-beginSafety],
-	    	    endParse-startPass2Safety, &prevChar, False, delimiters, string);
+                    &string[startPass2Safety-beginSafety],
+                    &styleString[startPass2Safety-beginSafety],
+                    endParse-startPass2Safety, &prevChar, delimiters, string,
+                    NULL);
 	    strncpy(&styleString[startPass2Safety-beginSafety], temp, tempLen);
 	    XtFree(temp);
 	}
@@ -1632,18 +1616,28 @@ parseDone:
 ** far the string should be searched, and "length" the part which is actually
 ** required (the string may or may not be parsed beyond "length").
 **
+** "lookBehindTo" indicates the boundary till where look-behind patterns may
+** look back. If NULL, the start of the string is assumed to be the boundary.
+**
+** "match_till" indicates the boundary till where matches may extend. If NULL,
+** it is assumed that the terminating \0 indicates the boundary. Note that
+** look-ahead patterns can peek beyond the boundary, if supplied.
+**
 ** Returns True if parsing was done and the parse succeeded.  Returns False if
 ** the error pattern matched, if the end of the string was reached without
 ** matching the end expression, or in the unlikely event of an internal error.
 */
-static int parseString(highlightDataRec *pattern, char **string,
+static int parseString(highlightDataRec *pattern, const char **string,
     	char **styleString, int length, char *prevChar, int anchored,
-    	const char *delimiters, const char* lookBehindTo)
+    	const char *delimiters, const char* lookBehindTo, 
+    	const char* match_till)
 {
     int i, subExecuted, subIndex;
-    char *stringPtr, *stylePtr, *startingStringPtr, *savedStartPtr;
+    char *stylePtr;
+    const char *stringPtr, *savedStartPtr, *startingStringPtr;
     signed char *subExpr;
     char savedPrevChar;
+    char succChar = match_till ? (*match_till) : '\0';
     highlightDataRec *subPat = NULL, *subSubPat;
     
     if (length <= 0)
@@ -1652,8 +1646,9 @@ static int parseString(highlightDataRec *pattern, char **string,
     stringPtr = *string;
     stylePtr = *styleString;
     
-    while(ExecRE(pattern->subPatternRE, NULL, stringPtr, anchored ? *string+1 :
-	    *string+length+1, False, *prevChar, '\0', delimiters, lookBehindTo)) {
+    while (ExecRE(pattern->subPatternRE, stringPtr, anchored ? *string+1 :
+	    *string+length+1, False, *prevChar, succChar, delimiters,
+            lookBehindTo, match_till)) {
 	/* Beware of the case where only one real branch exists, but that 
 	   branch has sub-branches itself. In that case the top_branch refers 
 	   to the matching sub-branch and must be ignored. */
@@ -1683,9 +1678,9 @@ static int parseString(highlightDataRec *pattern, char **string,
 		    subPat = pattern->subPatterns[i];
 		    if (subPat->colorOnly) {
 			if (!subExecuted) { 
-			    if (!ExecRE(pattern->endRE, NULL, savedStartPtr,
+                            if (!ExecRE(pattern->endRE, savedStartPtr,
 				savedStartPtr+1, False, savedPrevChar, 
-				'\0', delimiters, lookBehindTo)) {
+				succChar, delimiters, lookBehindTo, match_till)) {
 				fprintf(stderr, "Internal error, failed to "
 					"recover end match in parseString\n");
 				return False;
@@ -1734,17 +1729,31 @@ static int parseString(highlightDataRec *pattern, char **string,
     	    	    subPat->style, prevChar);
     	
     	/* Parse the remainder of the sub-pattern */	    
-    	} else {
-    	    
+    	} else if (subPat->endRE != NULL) {
+            /* The pattern is a starting/ending type of pattern, proceed with
+               the regular hierarchical parsing. */
+           
     	    /* If parsing should start after the start pattern, advance
-    	       to that point */
+    	       to that point (this is currently always the case) */
     	    if (!(subPat->flags & PARSE_SUBPATS_FROM_START))
     		fillStyleString(&stringPtr, &stylePtr, pattern->subPatternRE->endp[0], /* subPat->startRE->endp[0],*/
 			subPat->style, prevChar);
 
    	    /* Parse to the end of the subPattern */
    	    parseString(subPat, &stringPtr, &stylePtr, length -
-   	    	    (stringPtr - *string), prevChar, False, delimiters, lookBehindTo);
+   	    	    (stringPtr - *string), prevChar, False, delimiters,
+                    lookBehindTo, match_till);
+    	} else {
+    	    /* If the parent pattern is not a start/end pattern, the
+               sub-pattern can between the boundaries of the parent's 
+               match. Note that we must limit the recursive matches such
+               that they do not exceed the parent's ending boundary. 
+               Without that restriction, matching becomes unstable. */
+           
+   	    /* Parse to the end of the subPattern */
+   	    parseString(subPat, &stringPtr, &stylePtr, 
+                pattern->subPatternRE->endp[0]-stringPtr, prevChar, False, 
+                delimiters, lookBehindTo, pattern->subPatternRE->endp[0]);
     	}
     	
     	/* If the sub-pattern has color-only sub-sub-patterns, add color
@@ -1754,9 +1763,9 @@ static int parseString(highlightDataRec *pattern, char **string,
 	    subSubPat = subPat->subPatterns[i];
 	    if (subSubPat->colorOnly) {
 		if (!subExecuted) { 
-		    if (!ExecRE(subPat->startRE, NULL, savedStartPtr, 
-			savedStartPtr+1, False, savedPrevChar, '\0',
-			delimiters, lookBehindTo)) {
+                   if (!ExecRE(subPat->startRE, savedStartPtr, 
+			savedStartPtr+1, False, savedPrevChar, succChar,
+			delimiters, lookBehindTo, match_till)) {
 			fprintf(stderr, "Internal error, failed to recover "
 					"start match in parseString\n");
 			return False;
@@ -1805,12 +1814,12 @@ static int parseString(highlightDataRec *pattern, char **string,
 ** indirect and string pointers are not updated.
 */
 static void passTwoParseString(highlightDataRec *pattern, char *string,
-    	char *styleString, int length, char *prevChar, int anchored,
-    	const char *delimiters, const char *lookBehindTo)
+        char *styleString, int length, char *prevChar, const char *delimiters,
+        const char *lookBehindTo, const char *match_till)
 {
     int inParseRegion = False;
-    char *stylePtr, *stringPtr, temp, *parseStart = NULL, *parseEnd, *s, *c;
-    const 
+    char *stylePtr, temp, *parseStart = NULL, *parseEnd, *s, *c;
+    const char *stringPtr;
     int firstPass2Style = (unsigned char)pattern[1].style;
     
     for (c = string, s = styleString; ; c++, s++) {
@@ -1831,7 +1840,7 @@ static void passTwoParseString(highlightDataRec *pattern, char *string,
     	    /* printf("pass2 parsing %d chars\n", strlen(stringPtr)); */
     	    parseString(pattern, &stringPtr, &stylePtr,
     	    	    min(parseEnd - parseStart, length - (parseStart - string)),
-    	    	    prevChar, False, delimiters, lookBehindTo);
+    	    	    prevChar, False, delimiters, lookBehindTo, match_till);
     	    *parseEnd = temp;
     	    inParseRegion = False;
     	}
@@ -1846,8 +1855,8 @@ static void passTwoParseString(highlightDataRec *pattern, char *string,
 ** character, prevChar, which is fed to regular the expression matching
 ** routines for determining word and line boundaries at the start of the string.
 */
-static void fillStyleString(char **stringPtr, char **stylePtr, char *toPtr,
-    	char style, char *prevChar)
+static void fillStyleString(const char **stringPtr, char **stylePtr,
+    	const char *toPtr, char style, char *prevChar)
 {
     int i, len = toPtr-*stringPtr;
     
@@ -2328,10 +2337,11 @@ static int forwardOneContext(textBuffer *buf, reparseContext *context,
 ** sub-expression, "subExpr", of regular expression "re" applies to the
 ** corresponding portion of "string".
 */
-static void recolorSubexpr(regexp *re, int subexpr, int style, char *string,
-    	char *styleString)
+static void recolorSubexpr(regexp *re, int subexpr, int style,
+    	const char *string, char *styleString)
 {
-    char *stringPtr, *stylePtr;
+    const char *stringPtr;
+    char *stylePtr;
     	
     stringPtr = re->startp[subexpr];
     stylePtr = &styleString[stringPtr - string];
