@@ -1,4 +1,4 @@
-static const char CVSID[] = "$Id: nc.c,v 1.34.2.5 2003/11/20 18:37:13 edg Exp $";
+static const char CVSID[] = "$Id: nc.c,v 1.48.2.2 2009/11/30 20:21:18 lebert Exp $";
 /*******************************************************************************
 *									       *
 * nc.c -- Nirvana Editor client program for nedit server processes	       *
@@ -8,7 +8,8 @@ static const char CVSID[] = "$Id: nc.c,v 1.34.2.5 2003/11/20 18:37:13 edg Exp $"
 * This is free software; you can redistribute it and/or modify it under the    *
 * terms of the GNU General Public License as published by the Free Software    *
 * Foundation; either version 2 of the License, or (at your option) any later   *
-* version.							               *
+* version. In addition, you may distribute version of this program linked to   *
+* Motif or Open Motif. See README for details.                                 *
 * 									       *
 * This software is distributed in the hope that it will be useful, but WITHOUT *
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or        *
@@ -104,22 +105,23 @@ static void waitUntilFilesOpenedOrClosed(XtAppContext context,
                                          Window rootWindow);
 
 Display *TheDisplay;
+XtAppContext AppContext;
 static Atom currentWaitForAtom;
 static Atom noAtom = (Atom)(-1);
 
 static const char cmdLineHelp[] =
 #ifdef VMS
-"";
+"[Sorry, no on-line help available.]\n"; /* Why is that ? */
 #else
 "Usage:  nc [-read] [-create]\n"
 "           [-line n | +n] [-do command] [-lm languagemode]\n"
 "           [-svrname name] [-svrcmd command]\n"
 "           [-ask] [-noask] [-timeout seconds]\n"
 "           [-geometry geometry | -g geometry] [-icon | -iconic]\n"
-"           [-wait]\n"
-"           [-V | -version]\n"
+"           [-tabbed] [-untabbed] [-group] [-wait]\n"
+"           [-V | -version] [-h|-help]\n"
 "           [-xrm resourcestring] [-display [host]:server[.screen]]\n"
-"           [--] [file...]";
+"           [--] [file...]\n";
 #endif /*VMS*/
 
 /* Structure to hold X Resource values */
@@ -202,7 +204,7 @@ static void addToFileList(const char *path)
 }
 
 /* Creates the properties for the various paths */
-static void createWaitProperties()
+static void createWaitProperties(void)
 {
     FileListEntry *item;
 
@@ -234,7 +236,7 @@ int main(int argc, char **argv)
 
     /* Initialize toolkit and get an application context */
     XtToolkitInitialize();
-    context = XtCreateApplicationContext();
+    AppContext = context = XtCreateApplicationContext();
     
 #ifdef VMS
     /* Convert the command line to Unix style */
@@ -304,9 +306,8 @@ int main(int argc, char **argv)
                                       rootWindow,
                                       serverExistsAtom);
 
-    if (serverExists == False) {
+    if (serverExists == False)
         startNewServer(context, rootWindow, commandLine.shell, serverExistsAtom);
-    }
 
     waitUntilRequestProcessed(context,
                               rootWindow,
@@ -502,7 +503,7 @@ static int startServer(const char *message, const char *commandLineArgs)
     
     /* prompt user whether to start server */
     if (!Preferences.autoStart) {
-	puts(message);
+	printf(message);
 	do {
     	    c = getc(stdin);
 	} while (c == ' ' || c == '\t');
@@ -608,10 +609,10 @@ static void parseCommandLine(int argc, char **argv, CommandLine *commandLine)
     char name[MAXPATHLEN], path[MAXPATHLEN];
     const char *toDoCommand = "", *langMode = "", *geometry = "";
     char *commandString, *outPtr;
-    int lineNum = 0, read = 0, create = 0, iconic = 0, length = 0;
+    int lineNum = 0, read = 0, create = 0, iconic = 0, tabbed = -1, length = 0;
     int i, lineArg, nRead, charsWritten, opts = True;
-    int fileCount = 0;
-    
+    int fileCount = 0, group = 0, isTabbed;
+
     /* Allocate a string for output, for the maximum possible length.  The
        maximum length is calculated by assuming every argument is a file,
        and a complete record of maximum length is created for it */
@@ -649,6 +650,14 @@ static void parseCommandLine(int argc, char **argv, CommandLine *commandLine)
     	    read = 1;
     	} else if (opts && !strcmp(argv[i], "-create")) {
     	    create = 1;
+    	} else if (opts && !strcmp(argv[i], "-tabbed")) {
+    	    tabbed = 1;
+    	    group = 0;	/* override -group option */
+    	} else if (opts && !strcmp(argv[i], "-untabbed")) {
+    	    tabbed = 0;
+    	    group = 0;  /* override -group option */
+    	} else if (opts && !strcmp(argv[i], "-group")) {
+    	    group = 2; /* 2: start new group, 1: in group */
     	} else if (opts && (!strcmp(argv[i], "-iconic") || 
 	                    !strcmp(argv[i], "-icon"))) {
     	    iconic = 1;
@@ -679,6 +688,10 @@ static void parseCommandLine(int argc, char **argv, CommandLine *commandLine)
     	} else if (opts && (!strcmp(argv[i], "-version") || !strcmp(argv[i], "-V"))) {
     	    printNcVersion();
 	    exit(EXIT_SUCCESS);
+	} else if (opts && (!strcmp(argv[i], "-h") ||
+			    !strcmp(argv[i], "-help"))) {
+	    fprintf(stderr, "%s", cmdLineHelp);
+	    exit(EXIT_SUCCESS);
     	} else if (opts && (*argv[i] == '-')) {
 #ifdef VMS
 	    *argv[i] = '/';
@@ -693,6 +706,8 @@ static void parseCommandLine(int argc, char **argv, CommandLine *commandLine)
 	    /* Use VMS's LIB$FILESCAN for filename in argv[i] to process */
 	    /* wildcards and to obtain a full VMS file specification     */
 	    numFiles = VMSFileScan(argv[i], &nameList, NULL, INCLUDE_FNF);
+            /* Should we warn the user if he provided a -line or -do switch
+               and a wildcard pattern that expands to more than one file?  */
 	    /* for each expanded file name do: */
 	    for (j = 0; j < numFiles; ++j) {
 	    	oldLength = outPtr-commandString;
@@ -707,9 +722,23 @@ static void parseCommandLine(int argc, char **argv, CommandLine *commandLine)
 	           return;
 		}
     		strcat(path, name);
+
+		/* determine if file is to be openned in new tab, by
+		   factoring the options -group, -tabbed & -untabbed */
+    		if (group == 2) {
+	            isTabbed = 0;  /* start a new window for new group */
+		    group = 1;     /* next file will be within group */
+		} 
+		else if (group == 1) {
+	    	    isTabbed = 1;  /* new tab for file in group */
+		}
+		else {
+	    	    isTabbed = tabbed; /* not in group */
+		}
+	    
                 /* See below for casts */
-    		sprintf(outPtr, "%d %d %d %d %ld %ld %ld %ld\n%s\n%s\n%s\n%s\n%n",
-			lineNum, read, create, iconic, (long) strlen(path),
+    		sprintf(outPtr, "%d %d %d %d %d %ld %ld %ld %ld\n%s\n%s\n%s\n%s\n%n",
+			lineNum, read, create, iconic, tabbed, (long) strlen(path),
 			(long) strlen(toDoCommand), (long) strlen(langMode), 
                         (long) strlen(geometry),
 			path, toDoCommand, langMode, geometry, &charsWritten);
@@ -729,6 +758,20 @@ static void parseCommandLine(int argc, char **argv, CommandLine *commandLine)
 	       return;
 	    }
     	    strcat(path, name);
+	    
+	    /* determine if file is to be openned in new tab, by
+	       factoring the options -group, -tabbed & -untabbed */
+    	    if (group == 2) {
+	        isTabbed = 0;  /* start a new window for new group */
+		group = 1;     /* next file will be within group */
+	    } 
+	    else if (group == 1) {
+	    	isTabbed = 1;  /* new tab for file in group */
+	    }
+	    else {
+	    	isTabbed = tabbed; /* not in group */
+	    }
+	    
     	    /* SunOS 4 acc or acc and/or its runtime library has a bug
     	       such that %n fails (segv) if it follows a string in a
     	       printf or sprintf.  The silly code below avoids this.
@@ -736,10 +779,10 @@ static void parseCommandLine(int argc, char **argv, CommandLine *commandLine)
                The "long" cast on strlen() is necessary because size_t
                is 64 bit on Alphas, and 32-bit on most others.  There is
                no printf format specifier for "size_t", thanx, ANSI. */
-
-    	    sprintf(outPtr, "%d %d %d %d %ld %ld %ld %ld\n%n", lineNum, read,
-		    create, iconic, (long) strlen(path), (long) strlen(toDoCommand),
-		    (long) strlen(langMode), (long) strlen(geometry), &charsWritten);
+    	    sprintf(outPtr, "%d %d %d %d %d %ld %ld %ld %ld\n%n", lineNum,
+		    read, create, iconic, isTabbed, (long) strlen(path), 
+		    (long) strlen(toDoCommand), (long) strlen(langMode),
+		    (long) strlen(geometry), &charsWritten);
     	    outPtr += charsWritten;
     	    strcpy(outPtr, path);
     	    outPtr += strlen(path);
@@ -753,25 +796,29 @@ static void parseCommandLine(int argc, char **argv, CommandLine *commandLine)
     	    strcpy(outPtr, geometry);
     	    outPtr += strlen(geometry);
     	    *outPtr++ = '\n';
-	    toDoCommand = "";
 
             /* Create the file open atoms for the paths supplied */
             addToFileList(path);
 	    fileCount++;
 #endif /* VMS */
+
+            /* These switches only affect the next filename argument, not all */
+            toDoCommand = "";
+            lineNum = 0;
     	}
     }
 #ifdef VMS
     VMSFileScanDone();
 #endif /*VMS*/
     
-    /* If there's an un-written -do command,
+    /* If there's an un-written -do command, or we are to open a new window,
      * or user has requested iconic state, but not provided a file name,
      * create a server request with an empty file name and requested
      * iconic state (and optional language mode and geometry).
      */
     if (toDoCommand[0] != '\0' || fileCount == 0) {
-	sprintf(outPtr, "0 0 0 %d 0 %ld %ld %ld\n\n%n", iconic, (long) strlen(toDoCommand),
+	sprintf(outPtr, "0 0 0 %d %d 0 %ld %ld %ld\n\n%n", iconic, tabbed,
+		(long) strlen(toDoCommand),
 		(long) strlen(langMode), (long) strlen(geometry), &charsWritten);
 	outPtr += charsWritten;
 	strcpy(outPtr, toDoCommand);
@@ -955,7 +1002,7 @@ static void copyCommandLineArg(CommandLine *commandLine, const char *arg)
 /* Print version of 'nc' */
 static void printNcVersion(void ) {
    static const char *const ncHelpText = \
-   "nc (NEdit) Version 5.4 (November 2003)\n\n\
+   "nc (NEdit) Version 5.6 (November 2009)\n\n\
      Built on: %s, %s, %s\n\
      Built at: %s, %s\n";
      
