@@ -1,4 +1,4 @@
-/* $Id: nedit.h,v 1.58 2004/08/20 19:33:20 n8gray Exp $ */
+/* $Id: nedit.h,v 1.69.2.1 2010/07/07 20:47:12 lebert Exp $ */
 /*******************************************************************************
 *                                                                              *
 * nedit.h -- Nirvana Editor Common Header File                                 *
@@ -8,13 +8,13 @@
 * This is free software; you can redistribute it and/or modify it under the    *
 * terms of the GNU General Public License as published by the Free Software    *
 * Foundation; either version 2 of the License, or (at your option) any later   *
-* version. In addition, you may distribute version of this program linked to   *
+* version. In addition, you may distribute versions of this program linked to  *
 * Motif or Open Motif. See README for details.                                 *
 *                                                                              *
 * This software is distributed in the hope that it will be useful, but WITHOUT *
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or        *
-* FITNESS FOR A PARTICULAR PURPOSE.  See the GNU Lesser General Public License *
-* for more details.                                                            *
+* FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License for    *
+* more details.                                                                *
 *                                                                              *
 * You should have received a copy of the GNU General Public License along with *
 * software; if not, write to the Free Software Foundation, Inc., 59 Temple     *
@@ -29,6 +29,7 @@
 #define NEDIT_NEDIT_H_INCLUDED
 
 #include "textBuf.h"
+#include <sys/types.h>
 
 #include <X11/Intrinsic.h>
 #include <Xm/Xm.h>
@@ -41,6 +42,9 @@
 #endif
 #endif /*VMS*/
 
+#define NEDIT_VERSION           5
+#define NEDIT_REVISION          6
+
 /* Some default colors */
 #define NEDIT_DEFAULT_FG        "black"
 #define NEDIT_DEFAULT_TEXT_BG   "rgb:e5/e5/e5"
@@ -55,7 +59,7 @@
 
 
 /* Tuning parameters */
-#define SEARCHMAX 511		/* Maximum length of search/replace strings */
+#define SEARCHMAX 5119          /* Maximum length of search/replace strings */
 #define MAX_SEARCH_HISTORY 100	/* Maximum length of search string history */
 #define MAX_PANES 6		/* Max # of ADDITIONAL text editing panes
 				   that can be added to a window */
@@ -102,6 +106,10 @@ enum showMatchingStyle {NO_FLASH, FLASH_DELIMIT, FLASH_RANGE};
 enum virtKeyOverride { VIRT_KEY_OVERRIDE_NEVER, VIRT_KEY_OVERRIDE_AUTO,
                        VIRT_KEY_OVERRIDE_ALWAYS };
 
+/*  This enum must be kept in parallel to the array TruncSubstitutionModes[]
+    in preferences.c  */
+enum truncSubstitution {TRUNCSUBST_SILENT, TRUNCSUBST_FAIL, TRUNCSUBST_WARN, TRUNCSUBST_IGNORE};
+
 #define NO_FLASH_STRING		"off"
 #define FLASH_DELIMIT_STRING	"delimiter"
 #define FLASH_RANGE_STRING	"range"
@@ -346,7 +354,6 @@ typedef struct _WindowInfo {
     Widget  	noWrapItem;
     Widget  	newlineWrapItem;
     Widget  	continuousWrapItem;
-    Widget	showTabBarItem;
     Widget	statsLineItem;
     Widget	iSearchLineItem;
     Widget	lineNumsItem;
@@ -447,10 +454,14 @@ typedef struct _WindowInfo {
     char	filename[MAXPATHLEN];	/* name component of file being edited*/
     char	path[MAXPATHLEN];	/* path component of file being edited*/
     unsigned	fileMode;		/* permissions of file being edited */
+    uid_t	fileUid; 		/* last recorded user id of the file */
+    gid_t	fileGid;		/* last recorded group id of the file */
     int     	fileFormat; 	    	/* whether to save the file straight
     	    	    	    	    	   (Unix format), or convert it to
 					   MS DOS style with \r\n line breaks */
     time_t    	lastModTime; 	    	/* time of last modification to file */
+    dev_t       device;                 /*  device where the file resides */
+    ino_t       inode;                  /*  file's inode  */
     UndoInfo	*undo;			/* info for undoing last operation */
     UndoInfo	*redo;			/* info for redoing last undone op */
     textBuffer	*buffer;		/* holds the text being edited */
@@ -503,7 +514,6 @@ typedef struct _WindowInfo {
     					   and shell command executing modes */
     Boolean	ignoreModify;		/* ignore modifications to text area */
     Boolean	windowMenuValid;	/* is window menu up to date? */
-    Boolean	prevOpenMenuValid;	/* Prev. Opened Files menu up to date?*/
     int		rHistIndex, fHistIndex;	/* history placeholders for */
     int     	iSearchHistIndex;	/*   find and replace dialogs */
     int     	iSearchStartPos;    	/* start pos. of current incr. search */
