@@ -47,12 +47,12 @@ static int yygrowstack();
 #define ADD_SYM(sym) if (!AddSym(sym, &ErrMsg)) return 1
 #define ADD_IMMED(val) if (!AddImmediate(val, &ErrMsg)) return 1
 #define ADD_BR_OFF(to) if (!AddBranchOffset(to, &ErrMsg)) return 1
-#define SET_BR_OFF(from, to) *((int *)(from)) = ((Inst *)(to)) - ((Inst *)(from))
+#define SET_BR_OFF(from, to) ((from)->value) = ((Inst *)(to)) - ((Inst *)(from))
 
 /* Max. length for a string constant (... there shouldn't be a maximum) */
 #define MAX_STRING_CONST_LEN 5000
 
-static const char CVSID[] = "$Id: parse_noyacc.c,v 1.8 2003/12/19 23:23:31 slobasso Exp $";
+static const char CVSID[] = "$Id: parse_noyacc.c,v 1.10 2007/01/12 16:19:35 tringali Exp $";
 static int yyerror(char *s);
 static int yylex(void);
 int yyparse(void);
@@ -778,8 +778,12 @@ static int yylex(void)
     int i, len;
     Symbol *s;
     static DataValue value = {NO_TAG, {0}};
-    static char escape[] = "\\\"ntbrfav";
-    static char replace[] = "\\\"\n\t\b\r\f\a\v";
+    static char escape[] = "\\\"ntbrfave";
+#ifdef EBCDIC_CHARSET
+    static char replace[] = "\\\"\n\t\b\r\f\a\v\x27"; /* EBCDIC escape */
+#else
+    static char replace[] = "\\\"\n\t\b\r\f\a\v\x1B"; /* ASCII escape */
+#endif
 
     /* skip whitespace, backslash-newline combinations, and comments, which are
        all considered whitespace */
@@ -858,9 +862,18 @@ static int yylex(void)
         return SYMBOL;
     }
 
-    /* process quoted strings w/ embedded escape sequences */
+    /* Process quoted strings with embedded escape sequences:
+         For backslashes we recognise hexadecimal values with initial 'x' such
+       as "\x1B"; octal value (upto 3 oct digits with a possible leading zero)
+       such as "\33", "\033" or "\0033", and the C escapes: \", \', \n, \t, \b,
+       \r, \f, \a, \v, and the added \e for the escape character, as for REs.
+         Disallow hex/octal zero values (NUL): instead ignore the introductory
+       backslash, eg "\x0xyz" becomes "x0xyz" and "\0000hello" becomes
+       "0000hello". */
+
     if (*InPtr == '\"') {
         char string[MAX_STRING_CONST_LEN], *p = string;
+        char *backslash;
         InPtr++;
         while (*InPtr != '\0' && *InPtr != '\"' && *InPtr != '\n') {
             if (p >= string + MAX_STRING_CONST_LEN) {
@@ -868,23 +881,91 @@ static int yylex(void)
                 continue;
             }
             if (*InPtr == '\\') {
+                backslash = InPtr;
                 InPtr++;
                 if (*InPtr == '\n') {
                     InPtr++;
                     continue;
                 }
+                if (*InPtr == 'x') {
+                    /* a hex introducer */
+                    int hexValue = 0;
+                    const char *hexDigits = "0123456789abcdef";
+                    const char *hexD;
+                    InPtr++;
+                    if (*InPtr == '\0' ||
+                        (hexD = strchr(hexDigits, tolower(*InPtr))) == NULL) {
+                        *p++ = 'x';
+                    }
+                    else {
+                        hexValue = hexD - hexDigits;
+                        InPtr++;
+                        /* now do we have another digit? only accept one more */
+                        if (*InPtr != '\0' &&
+                            (hexD = strchr(hexDigits,tolower(*InPtr))) != NULL){
+                          hexValue = hexD - hexDigits + (hexValue << 4);
+                          InPtr++;
+                        }
+                        if (hexValue != 0) {
+                            *p++ = (char)hexValue;
+                        }
+                        else {
+                            InPtr = backslash + 1; /* just skip the backslash */
+                        }
+                    }
+                    continue;
+                }
+                /* the RE documentation requires \0 as the octal introducer;
+                   here you can start with any octal digit, but you are only
+                   allowed up to three (or four if the first is '0'). */
+                if ('0' <= *InPtr && *InPtr <= '7') {
+                    if (*InPtr == '0') {
+                        InPtr++;  /* octal introducer: don't count this digit */
+                    }
+                    if ('0' <= *InPtr && *InPtr <= '7') {
+                        /* treat as octal - first digit */
+                        char octD = *InPtr++;
+                        int octValue = octD - '0';
+                        if ('0' <= *InPtr && *InPtr <= '7') {
+                            /* second digit */
+                            octD = *InPtr++;
+                            octValue = (octValue << 3) + octD - '0';
+                            /* now do we have another digit? can we add it?
+                               if value is going to be too big for char (greater
+                               than 0377), stop converting now before adding the
+                               third digit */
+                            if ('0' <= *InPtr && *InPtr <= '7' &&
+                                octValue <= 037) {
+                                /* third digit is acceptable */
+                                octD = *InPtr++;
+                                octValue = (octValue << 3) + octD - '0';
+                            }
+                        }
+                        if (octValue != 0) {
+                            *p++ = (char)octValue;
+                        }
+                        else {
+                            InPtr = backslash + 1; /* just skip the backslash */
+                        }
+                    }
+                    else { /* \0 followed by non-digits: go back to 0 */
+                        InPtr = backslash + 1; /* just skip the backslash */
+                    }
+                    continue;
+                }
                 for (i=0; escape[i]!='\0'; i++) {
-                    if (escape[i] == '\0') {
-                        *p++= *InPtr++;
-                        break;
-                    } else if (escape[i] == *InPtr) {
+                    if (escape[i] == *InPtr) {
                         *p++ = replace[i];
                         InPtr++;
                         break;
                     }
                 }
-            } else
+                /* if we get here, we didn't recognise the character after
+                   the backslash: just copy it next time round the loop */
+            }
+            else {
                 *p++= *InPtr++;
+            }
         }
         *p = '\0';
         InPtr++;
@@ -994,7 +1075,7 @@ static int yyerror(char *s)
     ErrMsg = s;
     return 0;
 }
-#line 998 "y.tab.c"
+#line 1079 "y.tab.c"
 /* allocate initial stack or double stack size, up to YYMAXDEPTH */
 static int yygrowstack()
 {
@@ -1337,108 +1418,108 @@ break;
 case 29:
 #line 180 "parse.y"
 {
-                ADD_OP(OP_ARRAY_DELETE); ADD_IMMED((void *)yyvsp[-1].nArgs);
+                ADD_OP(OP_ARRAY_DELETE); ADD_IMMED(yyvsp[-1].nArgs);
             }
 break;
 case 30:
 #line 183 "parse.y"
 {
-                ADD_OP(OP_ARRAY_ASSIGN); ADD_IMMED((void *)yyvsp[-3].nArgs);
+                ADD_OP(OP_ARRAY_ASSIGN); ADD_IMMED(yyvsp[-3].nArgs);
             }
 break;
 case 31:
 #line 186 "parse.y"
 {
-                ADD_OP(OP_ARRAY_REF_ASSIGN_SETUP); ADD_IMMED((void *)1); ADD_IMMED((void *)yyvsp[-3].nArgs);
+                ADD_OP(OP_ARRAY_REF_ASSIGN_SETUP); ADD_IMMED(1); ADD_IMMED(yyvsp[-3].nArgs);
                 ADD_OP(OP_ADD);
-                ADD_OP(OP_ARRAY_ASSIGN); ADD_IMMED((void *)yyvsp[-3].nArgs);
+                ADD_OP(OP_ARRAY_ASSIGN); ADD_IMMED(yyvsp[-3].nArgs);
             }
 break;
 case 32:
 #line 191 "parse.y"
 {
-                ADD_OP(OP_ARRAY_REF_ASSIGN_SETUP); ADD_IMMED((void *)1); ADD_IMMED((void *)yyvsp[-3].nArgs);
+                ADD_OP(OP_ARRAY_REF_ASSIGN_SETUP); ADD_IMMED(1); ADD_IMMED(yyvsp[-3].nArgs);
                 ADD_OP(OP_SUB);
-                ADD_OP(OP_ARRAY_ASSIGN); ADD_IMMED((void *)yyvsp[-3].nArgs);
+                ADD_OP(OP_ARRAY_ASSIGN); ADD_IMMED(yyvsp[-3].nArgs);
             }
 break;
 case 33:
 #line 196 "parse.y"
 {
-                ADD_OP(OP_ARRAY_REF_ASSIGN_SETUP); ADD_IMMED((void *)1); ADD_IMMED((void *)yyvsp[-3].nArgs);
+                ADD_OP(OP_ARRAY_REF_ASSIGN_SETUP); ADD_IMMED(1); ADD_IMMED(yyvsp[-3].nArgs);
                 ADD_OP(OP_MUL);
-                ADD_OP(OP_ARRAY_ASSIGN); ADD_IMMED((void *)yyvsp[-3].nArgs);
+                ADD_OP(OP_ARRAY_ASSIGN); ADD_IMMED(yyvsp[-3].nArgs);
             }
 break;
 case 34:
 #line 201 "parse.y"
 {
-                ADD_OP(OP_ARRAY_REF_ASSIGN_SETUP); ADD_IMMED((void *)1); ADD_IMMED((void *)yyvsp[-3].nArgs);
+                ADD_OP(OP_ARRAY_REF_ASSIGN_SETUP); ADD_IMMED(1); ADD_IMMED(yyvsp[-3].nArgs);
                 ADD_OP(OP_DIV);
-                ADD_OP(OP_ARRAY_ASSIGN); ADD_IMMED((void *)yyvsp[-3].nArgs);
+                ADD_OP(OP_ARRAY_ASSIGN); ADD_IMMED(yyvsp[-3].nArgs);
             }
 break;
 case 35:
 #line 206 "parse.y"
 {
-                ADD_OP(OP_ARRAY_REF_ASSIGN_SETUP); ADD_IMMED((void *)1); ADD_IMMED((void *)yyvsp[-3].nArgs);
+                ADD_OP(OP_ARRAY_REF_ASSIGN_SETUP); ADD_IMMED(1); ADD_IMMED(yyvsp[-3].nArgs);
                 ADD_OP(OP_MOD);
-                ADD_OP(OP_ARRAY_ASSIGN); ADD_IMMED((void *)yyvsp[-3].nArgs);
+                ADD_OP(OP_ARRAY_ASSIGN); ADD_IMMED(yyvsp[-3].nArgs);
             }
 break;
 case 36:
 #line 211 "parse.y"
 {
-                ADD_OP(OP_ARRAY_REF_ASSIGN_SETUP); ADD_IMMED((void *)1); ADD_IMMED((void *)yyvsp[-3].nArgs);
+                ADD_OP(OP_ARRAY_REF_ASSIGN_SETUP); ADD_IMMED(1); ADD_IMMED(yyvsp[-3].nArgs);
                 ADD_OP(OP_BIT_AND);
-                ADD_OP(OP_ARRAY_ASSIGN); ADD_IMMED((void *)yyvsp[-3].nArgs);
+                ADD_OP(OP_ARRAY_ASSIGN); ADD_IMMED(yyvsp[-3].nArgs);
             }
 break;
 case 37:
 #line 216 "parse.y"
 {
-                ADD_OP(OP_ARRAY_REF_ASSIGN_SETUP); ADD_IMMED((void *)1); ADD_IMMED((void *)yyvsp[-3].nArgs);
+                ADD_OP(OP_ARRAY_REF_ASSIGN_SETUP); ADD_IMMED(1); ADD_IMMED(yyvsp[-3].nArgs);
                 ADD_OP(OP_BIT_OR);
-                ADD_OP(OP_ARRAY_ASSIGN); ADD_IMMED((void *)yyvsp[-3].nArgs);
+                ADD_OP(OP_ARRAY_ASSIGN); ADD_IMMED(yyvsp[-3].nArgs);
             }
 break;
 case 38:
 #line 221 "parse.y"
 {
-                ADD_OP(OP_ARRAY_REF_ASSIGN_SETUP); ADD_IMMED((void *)0); ADD_IMMED((void *)yyvsp[-2].nArgs);
+                ADD_OP(OP_ARRAY_REF_ASSIGN_SETUP); ADD_IMMED(0); ADD_IMMED(yyvsp[-2].nArgs);
                 ADD_OP(OP_INCR);
-                ADD_OP(OP_ARRAY_ASSIGN); ADD_IMMED((void *)yyvsp[-2].nArgs);
+                ADD_OP(OP_ARRAY_ASSIGN); ADD_IMMED(yyvsp[-2].nArgs);
             }
 break;
 case 39:
 #line 226 "parse.y"
 {
-                ADD_OP(OP_ARRAY_REF_ASSIGN_SETUP); ADD_IMMED((void *)0); ADD_IMMED((void *)yyvsp[-2].nArgs);
+                ADD_OP(OP_ARRAY_REF_ASSIGN_SETUP); ADD_IMMED(0); ADD_IMMED(yyvsp[-2].nArgs);
                 ADD_OP(OP_DECR);
-                ADD_OP(OP_ARRAY_ASSIGN); ADD_IMMED((void *)yyvsp[-2].nArgs);
+                ADD_OP(OP_ARRAY_ASSIGN); ADD_IMMED(yyvsp[-2].nArgs);
             }
 break;
 case 40:
 #line 231 "parse.y"
 {
-                ADD_OP(OP_ARRAY_REF_ASSIGN_SETUP); ADD_IMMED((void *)0); ADD_IMMED((void *)yyvsp[-1].nArgs);
+                ADD_OP(OP_ARRAY_REF_ASSIGN_SETUP); ADD_IMMED(0); ADD_IMMED(yyvsp[-1].nArgs);
                 ADD_OP(OP_INCR);
-                ADD_OP(OP_ARRAY_ASSIGN); ADD_IMMED((void *)yyvsp[-1].nArgs);
+                ADD_OP(OP_ARRAY_ASSIGN); ADD_IMMED(yyvsp[-1].nArgs);
             }
 break;
 case 41:
 #line 236 "parse.y"
 {
-                ADD_OP(OP_ARRAY_REF_ASSIGN_SETUP); ADD_IMMED((void *)0); ADD_IMMED((void *)yyvsp[-1].nArgs);
+                ADD_OP(OP_ARRAY_REF_ASSIGN_SETUP); ADD_IMMED(0); ADD_IMMED(yyvsp[-1].nArgs);
                 ADD_OP(OP_DECR);
-                ADD_OP(OP_ARRAY_ASSIGN); ADD_IMMED((void *)yyvsp[-1].nArgs);
+                ADD_OP(OP_ARRAY_ASSIGN); ADD_IMMED(yyvsp[-1].nArgs);
             }
 break;
 case 42:
 #line 241 "parse.y"
 {
                 ADD_OP(OP_SUBR_CALL);
-                ADD_SYM(PromoteToGlobal(yyvsp[-3].sym)); ADD_IMMED((void *)yyvsp[-1].nArgs);
+                ADD_SYM(PromoteToGlobal(yyvsp[-3].sym)); ADD_IMMED(yyvsp[-1].nArgs);
             }
 break;
 case 43:
@@ -1520,25 +1601,25 @@ break;
 case 56:
 #line 291 "parse.y"
 {
-                    ADD_OP(OP_PUSH_ARRAY_SYM); ADD_SYM(yyvsp[0].sym); ADD_IMMED((void *)1);
+                    ADD_OP(OP_PUSH_ARRAY_SYM); ADD_SYM(yyvsp[0].sym); ADD_IMMED(1);
                 }
 break;
 case 57:
 #line 294 "parse.y"
 {
-                    ADD_OP(OP_ARRAY_REF); ADD_IMMED((void *)yyvsp[-1].nArgs);
+                    ADD_OP(OP_ARRAY_REF); ADD_IMMED(yyvsp[-1].nArgs);
                 }
 break;
 case 58:
 #line 298 "parse.y"
 {
-                ADD_OP(OP_PUSH_ARRAY_SYM); ADD_SYM(yyvsp[0].sym); ADD_IMMED((void *)0);
+                ADD_OP(OP_PUSH_ARRAY_SYM); ADD_SYM(yyvsp[0].sym); ADD_IMMED(0);
             }
 break;
 case 59:
 #line 301 "parse.y"
 {
-                ADD_OP(OP_ARRAY_REF); ADD_IMMED((void *)yyvsp[-1].nArgs);
+                ADD_OP(OP_ARRAY_REF); ADD_IMMED(yyvsp[-1].nArgs);
             }
 break;
 case 60:
@@ -1569,7 +1650,7 @@ case 64:
 #line 318 "parse.y"
 {
                 ADD_OP(OP_SUBR_CALL);
-                ADD_SYM(PromoteToGlobal(yyvsp[-3].sym)); ADD_IMMED((void *)yyvsp[-1].nArgs);
+                ADD_SYM(PromoteToGlobal(yyvsp[-3].sym)); ADD_IMMED(yyvsp[-1].nArgs);
                 ADD_OP(OP_FETCH_RET_VAL);
             }
 break;
@@ -1594,7 +1675,7 @@ break;
 case 69:
 #line 333 "parse.y"
 {
-                ADD_OP(OP_ARRAY_REF); ADD_IMMED((void *)yyvsp[-1].nArgs);
+                ADD_OP(OP_ARRAY_REF); ADD_IMMED(yyvsp[-1].nArgs);
             }
 break;
 case 70:
@@ -1783,7 +1864,7 @@ case 99:
             ADD_BR_OFF(0);
         }
 break;
-#line 1787 "y.tab.c"
+#line 1868 "y.tab.c"
     }
     yyssp -= yym;
     yystate = *yyssp;
