@@ -1,4 +1,4 @@
-static const char CVSID[] = "$Id: shell.c,v 1.35 2004/09/02 08:49:56 edg Exp $";
+static const char CVSID[] = "$Id: shell.c,v 1.44 2008/01/04 22:11:04 yooden Exp $";
 /*******************************************************************************
 *									       *
 * shell.c -- Nirvana Editor shell command execution			       *
@@ -50,8 +50,10 @@ static const char CVSID[] = "$Id: shell.c,v 1.35 2004/09/02 08:49:56 edg Exp $";
 #include <signal.h>
 #include <sys/types.h>
 #ifndef __MVS__
+#ifndef VMS
 #include <sys/param.h>
 #endif
+#endif
 #include <sys/wait.h>
 #include <unistd.h>
 #include <fcntl.h>
@@ -317,7 +319,7 @@ void DoShellMenuCmd(WindowInfo *window, const char *command,
     int flags = 0;
     char *text;
     char *subsCommand, fullName[MAXPATHLEN];
-    int left, right, textLen;
+    int left = 0, right = 0, textLen;
     int pos, line, column;
     char lineNumber[11];
     WindowInfo *inWindow = window;
@@ -478,7 +480,7 @@ static void issueCommand(WindowInfo *window, const char *command, char *input,
 	int inputLen, int flags, Widget textW, int replaceLeft,
 	int replaceRight, int fromMacro)
 {
-    int stdinFD, stdoutFD, stderrFD;
+    int stdinFD, stdoutFD, stderrFD = 0;
     XtAppContext context = XtWidgetToApplicationContext(window->shell);
     shellCmdInfo *cmdData;
     pid_t childPid;
@@ -508,12 +510,12 @@ static void issueCommand(WindowInfo *window, const char *command, char *input,
     
     /* set the pipes connected to the process for non-blocking i/o */
     if (fcntl(stdinFD, F_SETFL, O_NONBLOCK) < 0)
-    	perror("NEdit: Internal error (fcntl)");
+    	perror("nedit: Internal error (fcntl)");
     if (fcntl(stdoutFD, F_SETFL, O_NONBLOCK) < 0)
-    	perror("NEdit: Internal error (fcntl1)");
+    	perror("nedit: Internal error (fcntl1)");
     if (flags & ERROR_DIALOGS) {
 	if (fcntl(stderrFD, F_SETFL, O_NONBLOCK) < 0)
-    	    perror("NEdit: Internal error (fcntl2)");
+    	    perror("nedit: Internal error (fcntl2)");
     }
     
     /* if there's nothing to write to the process' stdin, close it now */
@@ -592,7 +594,7 @@ static void stdoutReadProc(XtPointer clientData, int *source, XtInputId *id)
     /* error in read */
     if (nRead == -1) { /* error */
 	if (errno != EWOULDBLOCK && errno != EAGAIN) {
-	    perror("NEdit: Error reading shell command output");
+	    perror("nedit: Error reading shell command output");
 	    XtFree((char *)buf);
 	    finishCmdExecution(window, True);
 	}
@@ -633,7 +635,7 @@ static void stderrReadProc(XtPointer clientData, int *source, XtInputId *id)
     /* error in read */
     if (nRead == -1) {
 	if (errno != EWOULDBLOCK && errno != EAGAIN) {
-	    perror("NEdit: Error reading shell command error stream");
+	    perror("nedit: Error reading shell command error stream");
 	    XtFree((char *)buf);
 	    finishCmdExecution(window, True);
 	}
@@ -676,7 +678,7 @@ static void stdinWriteProc(XtPointer clientData, int *source, XtInputId *id)
     	    close(cmdData->stdinFD);
     	    cmdData->inPtr = NULL;
     	} else if (errno != EWOULDBLOCK && errno != EAGAIN) {
-    	    perror("NEdit: Write to shell command failed");
+    	    perror("nedit: Write to shell command failed");
     	    finishCmdExecution(window, True);
     	}
     } else {
@@ -695,14 +697,42 @@ static void stdinWriteProc(XtPointer clientData, int *source, XtInputId *id)
 ** Timer proc for putting up the "Shell Command in Progress" banner if
 ** the process is taking too long.
 */
+#define MAX_TIMEOUT_MSG_LEN (MAX_ACCEL_LEN + 60)
 static void bannerTimeoutProc(XtPointer clientData, XtIntervalId *id)
 {
     WindowInfo *window = (WindowInfo *)clientData;
     shellCmdInfo *cmdData = window->shellCmdData;
+    XmString xmCancel;
+    char* cCancel;
+    char message[MAX_TIMEOUT_MSG_LEN];
     
     cmdData->bannerIsUp = True;
-    SetModeMessage(window,
-    	    "Shell Command in Progress -- Press Ctrl+. to Cancel");
+
+    /* Extract accelerator text from menu PushButtons */
+    XtVaGetValues(window->cancelShellItem, XmNacceleratorText, &xmCancel, NULL);
+
+    /* Translate Motif string to char* */
+    cCancel = GetXmStringText(xmCancel);
+
+    /* Free Motif String */
+    XmStringFree(xmCancel);
+
+    /* Create message */
+    if ('\0' == cCancel[0])
+    {
+        strncpy(message, "Shell Command in Progress", MAX_TIMEOUT_MSG_LEN);
+        message[MAX_TIMEOUT_MSG_LEN - 1] = '\0';
+    } else
+    {
+        sprintf(message,
+                "Shell Command in Progress -- Press %s to Cancel",
+                cCancel);
+    }
+
+    /* Free C-string */
+    XtFree(cCancel);
+
+    SetModeMessage(window, message);
     cmdData->bannerTimeoutID = 0;
 }
 
@@ -746,7 +776,7 @@ static void flushTimeoutProc(XtPointer clientData, XtIntervalId *id)
 	    cmdData->leftPos += len;
 	    cmdData->rightPos = cmdData->leftPos;
 	} else
-	    fprintf(stderr, "NEdit: Too much binary data\n");
+	    fprintf(stderr, "nedit: Too much binary data\n");
     }
     XtFree(outText);
 
@@ -787,8 +817,7 @@ static void finishCmdExecution(WindowInfo *window, int terminatedOnError)
     	close(cmdData->stdinFD);
 
     /* Free the provided input text */
-    if (cmdData->input != NULL)
-	XtFree(cmdData->input);
+    XtFree(cmdData->input);
     
     /* Cancel pending timeouts */
     if (cmdData->flushTimeoutID != 0)
@@ -872,7 +901,7 @@ static void finishCmdExecution(WindowInfo *window, int terminatedOnError)
     } else {
 	buf = TextGetBuffer(cmdData->textW);
 	if (!BufSubstituteNullChars(outText, outTextLen, buf)) {
-	    fprintf(stderr,"NEdit: Too much binary data in shell cmd output\n");
+	    fprintf(stderr,"nedit: Too much binary data in shell cmd output\n");
 	    outText[0] = '\0';
 	}
 	if (cmdData->flags & REPLACE_SELECTION) {
@@ -923,13 +952,13 @@ static pid_t forkCommand(Widget parent, const char *command, const char *cmdDir,
        returned to the caller, the other half is spliced to stdin, stdout
        and stderr in the child process */
     if (pipe(pipeFDs) != 0) {
-    	perror("NEdit: Internal error (opening stdout pipe)");
+    	perror("nedit: Internal error (opening stdout pipe)");
         return -1;
     }
     *stdoutFD = pipeFDs[0];
     childStdoutFD = pipeFDs[1];
     if (pipe(pipeFDs) != 0) {
-    	perror("NEdit: Internal error (opening stdin pipe)");
+    	perror("nedit: Internal error (opening stdin pipe)");
         return -1;
     }
     *stdinFD = pipeFDs[1];
@@ -938,7 +967,7 @@ static pid_t forkCommand(Widget parent, const char *command, const char *cmdDir,
     	childStderrFD = childStdoutFD;
     else {
 	if (pipe(pipeFDs) != 0) {
-    	    perror("NEdit: Internal error (opening stdin pipe)");
+    	    perror("nedit: Internal error (opening stdin pipe)");
             return -1;
         }
 	*stderrFD = pipeFDs[0];
@@ -946,14 +975,17 @@ static pid_t forkCommand(Widget parent, const char *command, const char *cmdDir,
     }
     
     /* Fork the process */
+#ifdef VMS
+    childPid = vfork();
+#else
     childPid = fork();
+#endif
     
     /*
     ** Child process context (fork returned 0), clean up the
     ** child ends of the pipes and execute the command
     */
     if (0 == childPid) {
-
 	/* close the parent end of the pipes in the child process   */
 	close(*stdinFD);
 	close(*stdoutFD);
@@ -987,23 +1019,27 @@ static pid_t forkCommand(Widget parent, const char *command, const char *cmdDir,
 	/* make this process the leader of a new process group, so the sub
 	   processes can be killed, if necessary, with a killpg call */
 #ifndef __EMX__  /* OS/2 doesn't have this */
+#ifndef VMS  /* VMS doesn't have this */
 	setsid();
 #endif
-      
-       /* change the current working directory to the directory of the current
-	  file. */ 
-       if(cmdDir[0] != 0)
-	   if(chdir(cmdDir) == -1)
-	       perror("chdir to directory of current file failed");
+#endif
+     
+        /* change the current working directory to the directory of the
+            current file. */ 
+        if (cmdDir[0] != 0) {
+           if (chdir(cmdDir) == -1) {
+                perror("chdir to directory of current file failed");
+           }
+        }
      
 	/* execute the command using the shell specified by preferences */
-	execl(GetPrefShell(), GetPrefShell(), "-c", command, (char *)0);
+	execlp(GetPrefShell(), GetPrefShell(), "-c", command, NULL);
 
-	/* if we reach here, execl failed */
+	/* if we reach here, execlp failed */
 	fprintf(stderr, "Error starting shell: %s\n", GetPrefShell());
 	exit(EXIT_FAILURE);
     }
-    
+
     /* Parent process context, check if fork succeeded */
     if (childPid == -1)
     {
@@ -1130,7 +1166,7 @@ static void createOutputDialog(Widget parent, char *text)
     ac = 0;
     XtSetArg(al[ac], XmNlabelString, st1=MKSTRING("OK")); ac++;
     XtSetArg(al[ac], XmNmarginWidth, BUTTON_WIDTH_MARGIN); ac++;
-    XtSetArg(al[ac], XmNhighlightThickness, 0);  ac++;
+    XtSetArg(al[ac], XmNhighlightThickness, 2);  ac++;
     XtSetArg(al[ac], XmNbottomAttachment, XmATTACH_FORM);  ac++;
     XtSetArg(al[ac], XmNtopAttachment, XmATTACH_NONE);  ac++;
     button = XmCreatePushButtonGadget(form, "ok", al, ac);
@@ -1145,11 +1181,11 @@ static void createOutputDialog(Widget parent, char *text)
     XtSetArg(al[ac], XmNrows, rows);  ac++;
     XtSetArg(al[ac], XmNcolumns, cols);  ac++;
     XtSetArg(al[ac], XmNresizeHeight, False);  ac++;
-    XtSetArg(al[ac], XmNtraversalOn, False); ac++;
+    XtSetArg(al[ac], XmNtraversalOn, True); ac++;
     XtSetArg(al[ac], XmNwordWrap, True);  ac++;
     XtSetArg(al[ac], XmNscrollHorizontal, False);  ac++;
     XtSetArg(al[ac], XmNscrollVertical, hasScrollBar);  ac++;
-    XtSetArg(al[ac], XmNhighlightThickness, 0);  ac++;
+    XtSetArg(al[ac], XmNhighlightThickness, 2);  ac++;
     XtSetArg(al[ac], XmNspacing, 0);  ac++;
     XtSetArg(al[ac], XmNeditMode, XmMULTI_LINE_EDIT);  ac++;
     XtSetArg(al[ac], XmNeditable, False);  ac++;
@@ -1161,10 +1197,22 @@ static void createOutputDialog(Widget parent, char *text)
     XtSetArg(al[ac], XmNbottomWidget, button);  ac++;
     textW = XmCreateScrolledText(form, "outText", al, ac);
     AddMouseWheelSupport(textW);
+    MakeSingleLineTextW(textW); /* Binds <Return> to activate() */
     XtManageChild(textW);
     
     XtVaSetValues(XtParent(form), XmNtitle, "Output from Command", NULL);
     ManageDialogCenteredOnPointer(form);
+
+#ifdef LESSTIF_VERSION
+    /*
+     * The Lesstif text widget blocks activate() calls in multi-line mode,
+     * so we put the original focus on the Ok button such that the user
+     * can simply hit Return to dismiss the dialog. 
+     */
+    XmProcessTraversal(button, XmTRAVERSE_CURRENT);
+#else
+    XmProcessTraversal(textW, XmTRAVERSE_CURRENT);
+#endif
 }
 
 /*
