@@ -1,4 +1,4 @@
-static const char CVSID[] = "$Id: tags.c,v 1.59 2004/08/01 10:06:11 yooden Exp $";
+static const char CVSID[] = "$Id: tags.c,v 1.71 2009/06/23 21:30:09 lebert Exp $";
 /*******************************************************************************
 *                                                                              *
 * tags.c -- Nirvana editor tag file handling                                   *
@@ -8,7 +8,7 @@ static const char CVSID[] = "$Id: tags.c,v 1.59 2004/08/01 10:06:11 yooden Exp $
 * This is free software; you can redistribute it and/or modify it under the    *
 * terms of the GNU General Public License as published by the Free Software    *
 * Foundation; either version 2 of the License, or (at your option) any later   *
-* version. In addition, you may distribute version of this program linked to   *
+* version. In addition, you may distribute versions of this program linked to  *
 * Motif or Open Motif. See README for details.                                 *
 *                                                                              *
 * This software is distributed in the hope that it will be useful, but WITHOUT *
@@ -110,7 +110,7 @@ static void setTag(tag *t, const char *name, const char *file,
 static int fakeRegExSearch(WindowInfo *window, char *buffer, 
                         const char *searchString, int *startPos, int *endPos);
 static unsigned hashAddr(const char *key);
-static void updateMenuItems();
+static void updateMenuItems(void);
 static int addTag(const char *name, const char *file, int lang, 
                     const char *search, int posInf,  const  char *path, 
                     int index);
@@ -255,7 +255,6 @@ static int addTag(const char *name, const char *file, int lang,
         sprintf(newfile,"%s%s", path, file);
     
     NormalizePathname(newfile);
-    CompressPathname(newfile);
         
     for (t = table[addr]; t; t = t->next) {
         if (strcmp(name,t->name)) continue;
@@ -267,7 +266,6 @@ static int addTag(const char *name, const char *file, int lang,
             char tmpfile[MAXPATHLEN];
             sprintf(tmpfile, "%s%s", t->path, t->file);
             NormalizePathname(tmpfile);
-            CompressPathname(tmpfile);
             if (strcmp(newfile, tmpfile)) continue;
         }
         return 0;
@@ -368,7 +366,6 @@ int AddRelTagsFile(const char *tagSpec, const char *windowPath, int file_type)
         strcat(pathName, "/");
         strcat(pathName, filename);
         NormalizePathname(pathName);
-        CompressPathname(pathName);
 
         for (t = FileList; t && strcmp(t->filename, pathName); t = t->next);
         if (t) {
@@ -437,10 +434,13 @@ int AddTagsFile(const char *tagSpec, int file_type)
             strcpy(pathName,filename);
         }
         NormalizePathname(pathName);
-        CompressPathname(pathName);
 
         for (t = FileList; t && strcmp(t->filename,pathName); t = t->next);
         if (t) {
+            /* This file is already in the list.  It's easiest to just
+                refcount all tag/tip files even though we only actually care
+                about tip files. */
+            ++(t->refcount);
             added=1;
             continue;
         }
@@ -455,6 +455,7 @@ int AddTagsFile(const char *tagSpec, int file_type)
         t->date = statbuf.st_mtime;
         t->index = ++tagFileIndex;
         t->next = FileList;
+        t->refcount = 1;
         FileList = setFileListHead(t, file_type );
     }
     free(tmptagSpec);
@@ -468,8 +469,11 @@ int AddTagsFile(const char *tagSpec, int file_type)
 /* Un-manage a colon-delimited set of tags files 
  * Return TRUE if all files were found in the FileList and unloaded, FALSE
  * if any file was not found in the FileList.
+ * "file_type" is either TAG or TIP
+ * If "force_unload" is true, a calltips file will be deleted even if its
+ * refcount is nonzero.
  */
-int DeleteTagsFile(const char *tagSpec, int file_type)
+int DeleteTagsFile(const char *tagSpec, int file_type, Boolean force_unload)
 {   
     tagFile *t, *last;
     tagFile *FileList;
@@ -478,8 +482,7 @@ int DeleteTagsFile(const char *tagSpec, int file_type)
     
     /* To prevent any possible segfault */
     if (tagSpec == NULL) {
-        fprintf(stderr, "nedit: Internal Error!\n"
-                "  Passed NULL pointer to DeleteTagsFile!\n");
+        fprintf(stderr, "nedit: Internal Error: Passed NULL pointer to DeleteTagsFile!\n");
         return FALSE;
     }
     
@@ -502,11 +505,14 @@ int DeleteTagsFile(const char *tagSpec, int file_type)
             strcpy(pathName,filename);
         }
         NormalizePathname(pathName);
-        CompressPathname(pathName);
 
         for (last=NULL,t = FileList; t; last = t,t = t->next) {
             if (strcmp(t->filename, pathName))
                 continue;
+            /* Don't unload tips files with nonzero refcounts unless forced */
+            if (searchMode == TIP && !force_unload && --t->refcount > 0) {
+                break;
+            }
             if (t->loaded)
                 delTag(NULL,NULL,-2,NULL,-2,t->index);
             if (last) last->next = t->next;
@@ -530,7 +536,7 @@ int DeleteTagsFile(const char *tagSpec, int file_type)
 ** Update the "Find Definition", "Unload Tags File", "Show Calltip", 
 ** and "Unload Calltips File" menu items in the existing windows. 
 */
-static void updateMenuItems() 
+static void updateMenuItems(void) 
 {
     WindowInfo *w; 
     Boolean tipStat=FALSE, tagStat=FALSE;
@@ -869,11 +875,13 @@ static int findDef(WindowInfo *window, const char *value, int search_type) {
             tagText[ml] = '\0';
             /* See if we can find the tip/tag */
             status = findAllMatches(window, tagText);
+
             /* If we didn't find a requested calltip, see if we can use a tag */
             if (status == 0 && search_type == TIP && TagsFileList != NULL) {
                 searchMode = TIP_FROM_TAG;
                 status = findAllMatches(window, tagText);
             }
+
             if (status == 0) {
                 /* Didn't find any matches */
                 if (searchMode == TIP_FROM_TAG || searchMode == TIP) {
@@ -905,7 +913,7 @@ static int findDef(WindowInfo *window, const char *value, int search_type) {
 ** loaded tags file and bring up the file and line that the tags file
 ** indicates.
 */
-void findDefinitionHelper(WindowInfo *window, Time time, const char *arg,
+static void findDefinitionHelper(WindowInfo *window, Time time, const char *arg,
                    int search_type)
 {
     if(arg)
@@ -1010,12 +1018,12 @@ static int fakeRegExSearch(WindowInfo *window, char *in_buffer,
         const char *searchString, int *startPos, int *endPos)
 {
     int found, searchStartPos, dir, ctagsMode;
-    char *fileString, searchSubs[3*MAXLINE+3], *outPtr;
-    const char *inPtr;
+    char searchSubs[3*MAXLINE+3], *outPtr;
+    const char *fileString, *inPtr;
     
     if (in_buffer == NULL) {
         /* get the entire (sigh) text buffer from the text area widget */
-        fileString = BufGetAll(window->buffer);
+        fileString = BufAsString(window->buffer);
     } else {
         fileString = in_buffer;
     }
@@ -1036,8 +1044,6 @@ static int fakeRegExSearch(WindowInfo *window, char *in_buffer,
         ctagsMode=1;
     } else {
         fprintf(stderr, "NEdit: Error parsing tag file search string");
-        if(in_buffer == NULL)
-            XtFree(fileString);
         return FALSE;
     }
 
@@ -1093,10 +1099,6 @@ static int fakeRegExSearch(WindowInfo *window, char *in_buffer,
                 NULL, NULL);
     }
 
-    /* free the text buffer copy returned from XmTextGetString */
-    if(in_buffer == NULL)
-        XtFree(fileString);
-    
     /* return the result */
     if (found) {
         /* *startPos and *endPos are set in SearchString*/
@@ -1196,10 +1198,11 @@ static int findAllMatches(WindowInfo *window, const char *string)
     
     if (nMatches>1) {
         if (!(dupTagsList = (char **) malloc(sizeof(char *) * nMatches))) {
-            fprintf(stderr, "NEdit: findDef(): out of heap space!\n");
+            fprintf(stderr, "nedit: findAllMatches(): out of heap space!\n");
             XBell(TheDisplay, 0);
             return -1;
         }
+
         for (i=0; i<nMatches; i++) {
             ParseFilename(tagFiles[i], filename, pathname);
             if ((i<nMatches-1 && !strcmp(tagFiles[i],tagFiles[i+1])) ||
@@ -1214,13 +1217,24 @@ static int findAllMatches(WindowInfo *window, const char *string)
                     sprintf(temp,"%2d. %s%s %8i", i+1, pathname, filename,
                             tagPosInf[i]);
                 }
-            } else 
+            } else {
                 sprintf(temp,"%2d. %s%s",i+1,pathname,filename);
-            if (!(dupTagsList[i] = (char *) malloc(strlen(temp) + 1))) {
-                fprintf(stderr, "NEdit: findDef(): out of heap space!\n");
+            }
+
+            if (NULL == (dupTagsList[i] = (char*) malloc(strlen(temp) + 1))) {
+                int j;
+                fprintf(stderr, "nedit: findAllMatches(): out of heap space!\n");
+
+                /*  dupTagsList[i] is unallocated, let's free [i - 1] to [0]  */
+                for (j = i - 1; j > -1; j--) {
+                    free(dupTagsList[j]);
+                }
+                free(dupTagsList);
+
                 XBell(TheDisplay, 0);
                 return -1;
             }
+
             strcpy(dupTagsList[i],temp);
         }
         createSelectMenu(dialogParent, "Duplicate Tags", nMatches, dupTagsList);
@@ -1229,6 +1243,7 @@ static int findAllMatches(WindowInfo *window, const char *string)
         free(dupTagsList);
         return 1;
     }
+
     /*
     **  No need for a dialog list, there is only one tag matching --
     **  Go directly to the tag
@@ -1460,6 +1475,7 @@ static void editTaggedLocation( Widget parent, int i )
 
     /* select the matched string */
     BufSelect(windowToSearch->buffer, startPos, endPos);
+    RaiseFocusDocumentWindow(windowToSearch, True);
 
     /* Position it nicely in the window, 
        about 1/4 of the way down from the top */
@@ -1595,8 +1611,20 @@ static const char *rcs_strdup(const char *str)
     }
     else     /* Doesn't exist, conjure up a new one. */
     {
-        struct rcs *newrcs = malloc(sizeof(struct rcs));
-        newrcs->string = malloc(len+1);
+        struct rcs* newrcs;
+        if (NULL == (newrcs = (struct rcs*) malloc(sizeof(struct rcs)))) {
+            /*  Not much to fall back to here.  */
+            fprintf(stderr, "nedit: rcs_strdup(): out of heap space!\n");
+            XBell(TheDisplay, 0);
+            exit(1);
+        }
+
+        if (NULL == (newrcs->string = (char*) malloc(len + 1))) {
+            /*  Not much to fall back to here.  */
+            fprintf(stderr, "nedit: rcs_strdup(): out of heap space!\n");
+            XBell(TheDisplay, 0);
+            exit(1);
+        }
         strcpy(newrcs->string, str);
         newrcs->usage = 1;
         newrcs->next = NULL;
@@ -1783,9 +1811,9 @@ static int nextTFBlock(FILE *fp, char *header, char **body, int *blkLine,
         if (incPos < 0) 
             return TF_ERROR;
         /* Figure out how long the block is */
-        while((status=fgets(line, MAXLINE, fp))) {
+        while((status=fgets(line, MAXLINE, fp)) || feof(fp)) {
             ++(*currLine);
-            if(lineEmpty( line )) 
+            if(feof(fp) || lineEmpty( line ))
                 break;
         }
         incLen = ftell(fp) - incPos;
