@@ -1,4 +1,4 @@
-/* $Id: fileUtils.h,v 1.11 2004/07/21 11:32:07 yooden Exp $ */
+/* $Id: fileUtils.h,v 1.13 2008/08/20 14:57:36 lebert Exp $ */
 /*******************************************************************************
 *                                                                              *
 * fileUtils.h -- Nirvana Editor File Utilities Header File                     *
@@ -8,13 +8,13 @@
 * This is free software; you can redistribute it and/or modify it under the    *
 * terms of the GNU General Public License as published by the Free Software    *
 * Foundation; either version 2 of the License, or (at your option) any later   *
-* version. In addition, you may distribute version of this program linked to   *
+* version. In addition, you may distribute versions of this program linked to  *
 * Motif or Open Motif. See README for details.                                 *
 *                                                                              *
 * This software is distributed in the hope that it will be useful, but WITHOUT *
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or        *
-* FITNESS FOR A PARTICULAR PURPOSE.  See the GNU Lesser General Public License *
-* for more details.                                                            *
+* FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License for    *
+* more details.                                                                *
 *                                                                              *
 * You should have received a copy of the GNU General Public License along with *
 * software; if not, write to the Free Software Foundation, Inc., 59 Temple     *
@@ -44,6 +44,6 @@ void ConvertFromDosFileString(char *inString, int *length,
 void ConvertFromMacFileString(char *fileString, int length);
 int ConvertToDosFileString(char **fileString, int *length);
 void ConvertToMacFileString(char *fileString, int length);
-char *ReadAnyTextFile(const char *fileName);
+char *ReadAnyTextFile(const char *fileName, int forceNL);
 
 #endif /* NEDIT_FILEUTILS_H_INCLUDED */
