@@ -1,34 +1,52 @@
+static const char CVSID[] = "$Id: managedList.c,v 1.15 2006/08/08 10:59:32 edg Exp $";
 /*******************************************************************************
 *									       *
 * managedList.c -- User interface for reorderable list of records	       *
 *									       *
-* Copyright (c) 1991 Universities Research Association, Inc.		       *
-* All rights reserved.							       *
+* Copyright (C) 1999 Mark Edel						       *
+*									       *
+* This is free software; you can redistribute it and/or modify it under the    *
+* terms of the GNU General Public License as published by the Free Software    *
+* Foundation; either version 2 of the License, or (at your option) any later   *
+* version. In addition, you may distribute version of this program linked to   *
+* Motif or Open Motif. See README for details.                                 *
+*                                                                              *
+* This software is distributed in the hope that it will be useful, but WITHOUT *
+* ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or        *
+* FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License        *
+* for more details.							       *
 * 									       *
-* This material resulted from work developed under a Government Contract and   *
-* is subject to the following license:  The Government retaFins a paid-up,     *
-* nonexclusive, irrevocable worldwide license to reproduce, prepare derivative *
-* works, perform publicly and display publicly by or for the Government,       *
-* including the right to distribute to other Government contractors.  Neither  *
-* the United States nor the United States Department of Energy, nor any of     *
-* their employees, makes any warranty, express or implied, or assumes any      *
-* legal liability or responsibility for the accuracy, completeness, or         *
-* usefulness of any information, apparatus, product, or process disclosed, or  *
-* represents that its use would not infringe privately owned rights.           *
-*                                        				       *
-* Fermilab Nirvana GUI Library						       *
+* You should have received a copy of the GNU General Public License along with *
+* software; if not, write to the Free Software Foundation, Inc., 59 Temple     *
+* Place, Suite 330, Boston, MA  02111-1307 USA		                       *
+*									       *
+* Nirvana Text Editor	    						       *
 * November, 1995							       *
 *									       *
 * Written by Mark Edel							       *
 *									       *
 *******************************************************************************/
+
+#ifdef HAVE_CONFIG_H
+#include "../config.h"
+#endif
+
+#include "managedList.h"
+#include "misc.h"
+#include "DialogF.h"
+
 #include <stdio.h>
+#include <string.h>
+
+#include <X11/Intrinsic.h>
 #include <Xm/Form.h>
 #include <Xm/List.h>
 #include <Xm/PushB.h>
 #include <Xm/RowColumn.h>
-#include "misc.h"
-#include "managedList.h"
+
+#ifdef HAVE_DEBUG_H
+#include "../debug.h"
+#endif
 
 /* Common data between the managed list callback functions */
 typedef struct {
@@ -59,6 +77,7 @@ static void updateListWidgetItem(managedListData *ml, int listPos);
 static void listSelectionCB(Widget w, XtPointer clientData, XtPointer callData);
 static int selectedListPosition(managedListData *ml);
 static void selectItem(Widget listW, int itemIndex, int updateDialog);
+static Widget shellOfWidget(Widget w);
 
 /*
 ** Create a user interface to help manage a list of arbitrary data records
@@ -99,18 +118,18 @@ Widget CreateManagedList(Widget parent, char *name, Arg *args,
     	    XmNpacking, XmPACK_COLUMN,
     	    XmNleftAttachment, XmATTACH_FORM,
     	    XmNtopAttachment, XmATTACH_FORM,
-    	    XmNbottomAttachment, XmATTACH_FORM, 0);
+    	    XmNbottomAttachment, XmATTACH_FORM, NULL);
     deleteBtn = XtVaCreateManagedWidget("delete", xmPushButtonWidgetClass,
-    	    rowCol, XmNlabelString, s1=XmStringCreateSimple("Delete"), 0);
+    	    rowCol, XmNlabelString, s1=XmStringCreateSimple("Delete"), NULL);
     XmStringFree(s1);
     copyBtn = XtVaCreateManagedWidget("copy", xmPushButtonWidgetClass, rowCol,
-    	    XmNlabelString, s1=XmStringCreateSimple("Copy"), 0);
+    	    XmNlabelString, s1=XmStringCreateSimple("Copy"), NULL);
     XmStringFree(s1);
     moveUpBtn = XtVaCreateManagedWidget("moveUp", xmPushButtonWidgetClass,
-    	    rowCol, XmNlabelString, s1=XmStringCreateSimple("Move ^"), 0);
+    	    rowCol, XmNlabelString, s1=XmStringCreateSimple("Move ^"), NULL);
     XmStringFree(s1);
     moveDownBtn = XtVaCreateManagedWidget("moveDown", xmPushButtonWidgetClass,
-    	    rowCol, XmNlabelString, s1=XmStringCreateSimple("Move v"), 0);
+    	    rowCol, XmNlabelString, s1=XmStringCreateSimple("Move v"), NULL);
     XmStringFree(s1);
     
     /* AFAIK the only way to make a list widget n-columns wide is to make up
@@ -132,6 +151,7 @@ Widget CreateManagedList(Widget parent, char *name, Arg *args,
     XtSetArg(al[ac], XmNrightAttachment, XmATTACH_FORM); ac++;
     XtSetArg(al[ac], XmNbottomAttachment, XmATTACH_FORM); ac++;
     listW = XmCreateScrolledList(form, "list", al, ac);
+    AddMouseWheelSupport(listW);
     XtManageChild(listW);
     FreeStringTable(placeholderTable);
 
@@ -233,7 +253,7 @@ Widget ManageListAndButtons(Widget listW, Widget deleteBtn, Widget copyBtn,
     ml->moveUpBtn = moveUpBtn;
     ml->moveDownBtn = moveDownBtn;
     ml->getDialogDataCB = NULL;
-    ml->getDialogDataArg = setDialogDataArg;
+    ml->getDialogDataArg = getDialogDataArg;
     ml->setDialogDataCB = NULL;
     ml->setDialogDataArg = setDialogDataArg;
     ml->freeItemCB = freeItemCB;
@@ -246,7 +266,7 @@ Widget ManageListAndButtons(Widget listW, Widget deleteBtn, Widget copyBtn,
 
     /* Make the managed list data structure accessible from the list widget
        pointer, and make sure it gets freed when the list is destroyed */
-    XtVaSetValues(ml->listW, XmNuserData, ml, 0);
+    XtVaSetValues(ml->listW, XmNuserData, ml, NULL);
     XtAddCallback(ml->listW, XmNdestroyCallback, destroyCB, ml);
     
     /* Add callbacks for button and list actions */
@@ -278,7 +298,7 @@ int UpdateManagedList(Widget listW, int explicitRequest)
     
     /* Recover the pointer to the managed list structure from the widget's
        userData pointer */
-    XtVaGetValues(listW, XmNuserData, &ml, 0);
+    XtVaGetValues(listW, XmNuserData, &ml, NULL);
     
     /* Make the update */
     return incorporateDialogData(ml, selectedListPosition(ml), explicitRequest);
@@ -294,7 +314,7 @@ void ChangeManagedListData(Widget listW)
     
     /* Recover the pointer to the managed list structure from the widget's
        userData pointer */
-    XtVaGetValues(listW, XmNuserData, &ml, 0);
+    XtVaGetValues(listW, XmNuserData, &ml, NULL);
     
     updateDialogFromList(ml, -1);
 }
@@ -315,7 +335,7 @@ int ManagedListSelectedIndex(Widget listW)
 {
     managedListData *ml;
     
-    XtVaGetValues(listW, XmNuserData, &ml, 0);
+    XtVaGetValues(listW, XmNuserData, &ml, NULL);
     return selectedListPosition(ml)-2;
 }
 
@@ -329,7 +349,7 @@ void AddDeleteConfirmCB(Widget listW, int (*deleteConfirmCB)(int, void *),
 {
     managedListData *ml;
     
-    XtVaGetValues(listW, XmNuserData, &ml, 0);
+    XtVaGetValues(listW, XmNuserData, &ml, NULL);
     ml->deleteConfirmCB = deleteConfirmCB;
     ml->deleteConfirmArg = deleteConfirmArg;
 }
@@ -349,27 +369,27 @@ static void destroyCB(Widget w, XtPointer clientData, XtPointer callData)
 static void deleteCB(Widget w, XtPointer clientData, XtPointer callData)
 {
     managedListData *ml = (managedListData *)clientData;
-    int i, index, listPos;
+    int i, ind, listPos;
         
     /* get the selected list position and the item to be deleted */
     listPos = selectedListPosition(ml);
-    index = listPos-2;
+    ind = listPos-2;
     
     /* if there's a delete confirmation callback, call it first, and allow
        it to request that the operation be aborted */
     if (ml->deleteConfirmCB != NULL)
-    	if (!(*ml->deleteConfirmCB)(index, ml->deleteConfirmArg))
+    	if (!(*ml->deleteConfirmCB)(ind, ml->deleteConfirmArg))
     	    return;
 
     /* free the item and remove it from the list */
-    (*ml->freeItemCB)(ml->itemList[index]);
-    for (i=index; i<*ml->nItems-1; i++)
+    (*ml->freeItemCB)(ml->itemList[ind]);
+    for (i=ind; i<*ml->nItems-1; i++)
     	ml->itemList[i] = ml->itemList[i+1];
     (*ml->nItems)--;
     
     /* update the list widget and move the selection to the previous item
        in the list and display the fields appropriate  for that entry */
-    updateDialogFromList(ml, index-1);
+    updateDialogFromList(ml, ind-1);
 }
 
 static void copyCB(Widget w, XtPointer clientData, XtPointer callData)
@@ -383,6 +403,13 @@ static void copyCB(Widget w, XtPointer clientData, XtPointer callData)
     if (listPos == 1)
     	return; /* can't copy "new" */
 
+    if ((*ml->nItems) == ml->maxItems) {
+        DialogF(DF_ERR, shellOfWidget(ml->listW), 1, "Limits exceeded",
+                        "Cannot copy item.\nToo many items in list.",
+                        "OK");
+        return;
+    }
+
     /* Bring the entry up to date (could result in operation being canceled) */
     item = (*ml->getDialogDataCB)(ml->itemList[listPos-2], False, &abort,
     	    ml->getDialogDataArg);
@@ -393,9 +420,16 @@ static void copyCB(Widget w, XtPointer clientData, XtPointer callData)
 	ml->itemList[listPos-2] = item;
     }
     
-    /* Make a copy by requesting the data again */
+    /* Make a copy by requesting the data again.
+       In case getDialogDataCB() returned a fallback value, the dialog may
+       not be in sync with the internal list. If we _explicitly_ request the
+       data again, we could get an invalid answer. Therefore, we first update
+       the dialog to make sure that we can copy the right data. */
+    updateDialogFromList(ml, listPos-2);
     item = (*ml->getDialogDataCB)(ml->itemList[listPos-2], True, &abort,
     	    ml->getDialogDataArg);
+    if (abort)
+	return;
         
     /* add the item to the item list */
     for (i= *ml->nItems; i>=listPos; i--)
@@ -410,12 +444,12 @@ static void copyCB(Widget w, XtPointer clientData, XtPointer callData)
 static void moveUpCB(Widget w, XtPointer clientData, XtPointer callData)
 {
     managedListData *ml = (managedListData *)clientData;
-    int index, listPos;
+    int ind, listPos;
     void *temp;
         
     /* get the item index currently selected in the menu item list */
     listPos = selectedListPosition(ml);
-    index = listPos-2;
+    ind = listPos-2;
      
     /* Bring the item up to date with the dialog fields (It would be better
        if this could be avoided, because user errors will be flagged here,
@@ -424,23 +458,23 @@ static void moveUpCB(Widget w, XtPointer clientData, XtPointer callData)
     	return;
      
     /* shuffle the item up in the menu item list */
-    temp = ml->itemList[index];
-    ml->itemList[index] = ml->itemList[index-1];
-    ml->itemList[index-1] = temp;
+    temp = ml->itemList[ind];
+    ml->itemList[ind] = ml->itemList[ind-1];
+    ml->itemList[ind-1] = temp;
     
     /* update the list widget and keep the selection on moved item */
-    updateDialogFromList(ml, index-1);
+    updateDialogFromList(ml, ind-1);
 }
 
 static void moveDownCB(Widget w, XtPointer clientData, XtPointer callData)
 {
     managedListData *ml = (managedListData *)clientData;
-    int index, listPos;
+    int ind, listPos;
     void *temp;
         
     /* get the item index currently selected in the menu item list */
     listPos = selectedListPosition(ml);
-    index = listPos-2;
+    ind = listPos-2;
 
     /* Bring the item up to date with the dialog fields (I wish this could
        be avoided) */
@@ -448,12 +482,12 @@ static void moveDownCB(Widget w, XtPointer clientData, XtPointer callData)
     	return;
      
     /* shuffle the item down in the menu item list */
-    temp = ml->itemList[index];
-    ml->itemList[index] = ml->itemList[index+1];
-    ml->itemList[index+1] = temp;
+    temp = ml->itemList[ind];
+    ml->itemList[ind] = ml->itemList[ind+1];
+    ml->itemList[ind+1] = temp;
     
     /* update the list widget and keep the selection on moved item */
-    updateDialogFromList(ml, index+1);
+    updateDialogFromList(ml, ind+1);
 }
 
 /*
@@ -462,7 +496,7 @@ static void moveDownCB(Widget w, XtPointer clientData, XtPointer callData)
 static void listSelectionCB(Widget w, XtPointer clientData, XtPointer callData)
 {
     managedListData *ml = (managedListData *)clientData;
-    int index, listPos = ((XmListCallbackStruct *)callData)->item_position;
+    int ind, listPos = ((XmListCallbackStruct *)callData)->item_position;
     
     /* Save the current dialog fields before overwriting them.  If there's an
        error, force the user to go back to the old selection and fix it
@@ -493,11 +527,11 @@ static void listSelectionCB(Widget w, XtPointer clientData, XtPointer callData)
     }
         
     /* get the index of the item currently selected in the item list */
-    index = listPos - 2;
+    ind = listPos - 2;
     
     /* tell the caller to show the new item */
     if (ml->setDialogDataCB != NULL)
-    	(*ml->setDialogDataCB)(listPos==1 ? NULL : ml->itemList[index],
+    	(*ml->setDialogDataCB)(listPos==1 ? NULL : ml->itemList[ind],
     	    	ml->setDialogDataArg);
 }
 
@@ -525,6 +559,12 @@ static int incorporateDialogData(managedListData *ml, int listPos, int explicit)
     /* If the item is "new" add a new entry to the list, otherwise,
        modify the entry with the text fields from the dialog */
     if (listPos == 1) {
+        if ((*ml->nItems) == ml->maxItems) {
+            DialogF(DF_ERR, shellOfWidget(ml->listW), 1, "Limits exceeded",
+                            "Cannot add new item.\nToo many items in list.",
+                            "OK");
+            return False;
+        }
 	ml->itemList[(*ml->nItems)++] = item;
 	updateDialogFromList(ml, *ml->nItems - 1);
     } else {
@@ -555,7 +595,7 @@ static void updateDialogFromList(managedListData *ml, int selection)
     for (i=0; i < *ml->nItems; i++)
     	stringTable[i+1] = XmStringCreateSimple(*(char **)ml->itemList[i]);
     XtVaSetValues(ml->listW, XmNitems, stringTable,
-    	    XmNitemCount, *ml->nItems+1, 0);
+    	    XmNitemCount, *ml->nItems+1, NULL);
     for (i=0; i < *ml->nItems+1; i++)
     	XmStringFree(stringTable[i]);
     XtFree((char *)stringTable);
@@ -627,9 +667,18 @@ static void selectItem(Widget listW, int itemIndex, int updateDialog)
     
     /* If the selected item is not visible, scroll the list */
     XtVaGetValues(listW, XmNtopItemPosition, &topPos, XmNvisibleItemCount,
-    	    &nVisible, 0);
+    	    &nVisible, NULL);
     if (selection < topPos)
     	XmListSetPos(listW, selection);
     else if (selection >= topPos + nVisible)
     	XmListSetPos(listW, selection - nVisible + 1);
 }
+
+static Widget shellOfWidget(Widget w)
+{
+    while(1) {
+        if (!w) return 0;
+        if (XtIsSubclass(w, shellWidgetClass)) return w;
+        w = XtParent(w);
+    }      
+}
