@@ -1,4 +1,4 @@
-static const char CVSID[] = "$Id: printUtils.c,v 1.11 2001/08/14 08:37:16 jlous Exp $";
+static const char CVSID[] = "$Id: printUtils.c,v 1.25 2004/08/01 10:06:12 yooden Exp $";
 /*******************************************************************************
 *									       *
 * printUtils.c -- Nirvana library Printer Menu	& Printing Routines   	       *
@@ -8,8 +8,9 @@ static const char CVSID[] = "$Id: printUtils.c,v 1.11 2001/08/14 08:37:16 jlous
 * This is free software; you can redistribute it and/or modify it under the    *
 * terms of the GNU General Public License as published by the Free Software    *
 * Foundation; either version 2 of the License, or (at your option) any later   *
-* version.							               *
-* 									       *
+* version. In addition, you may distribute version of this program linked to   *
+* Motif or Open Motif. See README for details.                                 *
+*                                                                              *
 * This software is distributed in the hope that it will be useful, but WITHOUT *
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or        *
 * FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License        *
@@ -31,12 +32,22 @@ static const char CVSID[] = "$Id: printUtils.c,v 1.11 2001/08/14 08:37:16 jlous
 * Modified by Donna Reid and Joy Kyriakopulos 4/8/93 - VMS port		       *
 *									       *
 *******************************************************************************/
+
+#ifdef HAVE_CONFIG_H
+#include "../config.h"
+#endif
+
+#include "printUtils.h"
+#include "DialogF.h"
+#include "misc.h"
+#include "prefFile.h"
+
 #include <string.h>
 #include <errno.h>
 #include <stdio.h>
 #include <stdlib.h>
-#include <sys/types.h>
 #include <ctype.h>
+#include <sys/types.h>
 #ifdef VMS
 #include "vmsparam.h"
 #include <ssdef.h>
@@ -56,6 +67,7 @@ static const char CVSID[] = "$Id: printUtils.c,v 1.11 2001/08/14 08:37:16 jlous
 #endif
 #endif /*VMS*/
 #include <sys/stat.h>
+
 #include <X11/StringDefs.h>
 #include <X11/Intrinsic.h>
 #include <X11/Shell.h>
@@ -65,10 +77,10 @@ static const char CVSID[] = "$Id: printUtils.c,v 1.11 2001/08/14 08:37:16 jlous
 #include <Xm/PushB.h>
 #include <Xm/SeparatoG.h>
 #include <Xm/Text.h>
-#include "DialogF.h"
-#include "misc.h"
-#include "prefFile.h"
-#include "printUtils.h"
+
+#ifdef HAVE_DEBUG_H
+#include "../debug.h"
+#endif
 
 /* Separator between directory references in PATH environmental variable */
 #ifdef __EMX__  /* For OS/2 */
@@ -115,7 +127,9 @@ static void setQueueLabelText(void);
 static int fileInDir(const char *filename, const char *dirpath, unsigned short mode_flags);
 static int fileInPath(const char *filename, unsigned short mode_flags);
 static int flprPresent(void);
+#ifdef USE_LPR_PRINT_CMD
 static void getLprQueueDefault(char *defqueue);
+#endif
 #ifndef USE_LPR_PRINT_CMD
 static void getLpQueueDefault(char *defqueue);
 #endif
@@ -232,8 +246,8 @@ void PrintFile(Widget parent, const char *printFile, const char *jobName)
 **			(flpr is a Fermilab utility for printing on
 **			arbitrary systems that support the lpr protocol)
 */
-void LoadPrintPreferences(XrmDatabase prefDB, char *appName, char *appClass,
-	int lookForFlpr)
+void LoadPrintPreferences(XrmDatabase prefDB, const char *appName,
+        const char *appClass, int lookForFlpr)
 {
     static char defaultQueue[MAX_QUEUE_STR], defaultHost[MAX_HOST_STR];
 
@@ -309,7 +323,7 @@ static Widget createForm(Widget parent)
     XtSetArg(args[argcnt], XmNdialogStyle, XmDIALOG_FULL_APPLICATION_MODAL);
     argcnt++;
     XtSetArg(args[argcnt], XmNdialogTitle, (st0=XmStringCreateLtoR(
-		"Printer Options", XmSTRING_DEFAULT_CHARSET))); argcnt++;
+		"Print", XmSTRING_DEFAULT_CHARSET))); argcnt++;
     XtSetArg(args[argcnt], XmNautoUnmanage, False); argcnt++;
     form = CreateFormDialog(parent, "printForm", args, argcnt);
     XtVaSetValues(form, XmNshadowThickness, 0, NULL);
@@ -554,7 +568,7 @@ static void allowOnlyNumInput(Widget widget, caddr_t client_data,
     textInserted = (nInserted > 0);
     if ((call_data->reason == XmCR_MODIFYING_TEXT_VALUE) && textInserted) {
 	for (i=0; i<nInserted; i++) {
-            if (!isdigit(call_data->text->ptr[i])) {
+            if (!isdigit((unsigned char)call_data->text->ptr[i])) {
             	call_data->doit = False;
             	return;
             }
@@ -581,7 +595,7 @@ static void noSpaceOrPunct(Widget widget, caddr_t client_data,
     textInserted = (nInserted > 0);
     if ((call_data->reason == XmCR_MODIFYING_TEXT_VALUE) && textInserted) {
 	for (i=0; i<nInserted; i++) {
-            for (j=0; j<XtNumber(prohibited); j++) {
+            for (j=0; j<(int)XtNumber(prohibited); j++) {
         	if (call_data->text->ptr[i] == prohibited[j]) {
             	    call_data->doit = False;
             	    return;
@@ -699,17 +713,18 @@ static void printButtonCB(Widget widget, caddr_t client_data, caddr_t call_data)
     cmdDesc.dsc$b_class   = DSC$K_CLASS_S;
     cmdDesc.dsc$a_pointer = command;
     spawn_sts = lib$spawn(&cmdDesc,0,0,&spawnFlags,0,0,0,0,0,0,0,0);
-    if (spawn_sts != SS$_NORMAL) {
-       	DialogF(DF_WARN, widget, 1,
-       		"Unable to Print:\n%d - %s\n  spawnFlags = %d\n",
-       		"Dismiss", spawn_sts, strerror(EVMSERR, spawn_sts),
-       		spawnFlags);
+
+    if (spawn_sts != SS$_NORMAL)
+    {
+        DialogF(DF_WARN, widget, 1, "Print Error",
+                "Unable to Print:\n%d - %s\n  spawnFlags = %d\n", "OK",
+                spawn_sts, strerror(EVMSERR, spawn_sts), spawnFlags);
         return;
     }
 #else
     int nRead;
     FILE *pipe;
-    char errorString[MAX_PRINT_ERROR_LENGTH];
+    char errorString[MAX_PRINT_ERROR_LENGTH], discarded[1024];
 
     /* get the print command from the command text area */
     str = XmTextGetString(Text4);
@@ -720,20 +735,31 @@ static void printButtonCB(Widget widget, caddr_t client_data, caddr_t call_data)
     
     /* Issue the print command using a popen call and recover error messages
        from the output stream of the command. */
-    pipe = (FILE *)popen(command,"r");
-    if (pipe == NULL) {
-	DialogF(DF_WARN, widget, 1, "Unable to Print:\n%s",
-       		"Dismiss", strerror(errno));
-	return;
+    pipe = popen(command,"r");
+    if (pipe == NULL)
+    {
+        DialogF(DF_WARN, widget, 1, "Print Error", "Unable to Print:\n%s",
+                "OK", strerror(errno));
+        return;
     }
+
     errorString[0] = 0;
     nRead = fread(errorString, sizeof(char), MAX_PRINT_ERROR_LENGTH-1, pipe);
+    /* Make sure that the print command doesn't get stuck when trying to 
+       write a lot of output on stderr (pipe may fill up). We discard 
+       the additional output, though. */
+    while (fread(discarded, sizeof(char), 1024, pipe) > 0);
+
     if (!ferror(pipe))
-	errorString[nRead] = '\0';
-    if (pclose(pipe)) {
-	DialogF(DF_WARN, widget, 1, "Unable to Print:\n%s",
-       		"Dismiss", errorString);
-	return;
+    {
+        errorString[nRead] = '\0';
+    }
+
+    if (pclose(pipe))
+    {
+        DialogF(DF_WARN, widget, 1, "Print Error", "Unable to Print:\n%s",
+                "OK", errorString);
+        return;
     }
 #endif /*(VMS)*/
     
@@ -896,11 +922,13 @@ static void getFlprQueueDefault(char *defqueue)
             strcpy(defqueue,"");
 }
 
+#ifdef USE_LPR_PRINT_CMD
 static void getLprQueueDefault(char *defqueue)
 {
     if (!foundEnv("PRINTER",defqueue))
         strcpy(defqueue,"");
 }
+#endif
 
 #ifndef USE_LPR_PRINT_CMD
 static void getLpQueueDefault(char *defqueue)
@@ -947,19 +975,24 @@ static void getVmsQueueDefault(char *defqueue)
     translStruct.end_entry = 0;
     translStruct.queName = defqueue;
     translate_sts = sys$trnlnm(0,&tabName,&logName,0,&translStruct);
+
     if (translate_sts != SS$_NORMAL && translate_sts != SS$_NOLOGNAM){
-       fprintf(stderr,"Error return from sys$trnlnm: %d\n", translate_sts);
-       DialogF(DF_WARN, Label2, 1, "Error translating SYS$PRINT", "Dismiss");
+       fprintf(stderr, "Error return from sys$trnlnm: %d\n", translate_sts);
+       DialogF(DF_WARN, Label2, 1, "Error", "Error translating SYS$PRINT",
+               "OK");
        defqueue[0] = '\0';
-    }
-    else
+    } else
+    {
 /*    printf("return status from sys$trnlnm = %d\n", translate_sts); */
-    if (translate_sts == SS$_NOLOGNAM)
-	defqueue[0] = '\0';
-    else {
-	strncpy(defqueue, translStruct.queName, ret_len);
-	defqueue[ret_len] = '\0';
-/*	printf("defqueue = %s, length = %d\n", defqueue, ret_len); */
+        if (translate_sts == SS$_NOLOGNAM)
+        {
+            defqueue[0] = '\0';
+        } else
+        {
+            strncpy(defqueue, translStruct.queName, ret_len);
+            defqueue[ret_len] = '\0';
+/*  printf("defqueue = %s, length = %d\n", defqueue, ret_len); */
+        }
     }   
 }
-#endif
+#endif /*(VMS)*/
